<?php

/*
Copyright (c) 2007 Alex Tingle.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

define('EXA_VERSION','1.0.beta-1');
define('EXA_URL','http://blog.firetree.net/2007/04/04/easy-ajax-album-10beta-1/');

?>
<?php // utilities.php

/** Utility, truncates $x to within $a..$b. */
function minmax($a,$b,$x)
{
  return max($a,min($b,$x));
}

/** Utility, returns $p wrapped to be within 0..($l-1). */
function normalize($p,$l)
{
  return intval($p+$l) % $l;
}

/** Utility function. Gets the value of a variable from a cookie or
 *  a _GET var. In the latter case, set the cookie to match the new value. */
function get_set_cookie($name,$min=0,$max=1,$dflt=0)
{
  if(array_key_exists($name,$_GET))
  {
    $result=minmax($min,$max,intval($_GET[$name]));
    setcookie('exa_'.$name,$result,0,'/');  
  }
  elseif(array_key_exists('exa_'.$name,$_COOKIE))
  {
    $result=minmax($min,$max,intval($_COOKIE['exa_'.$name]));
  }
  else
  {
    $result=$dflt;
  }
  return $result;
}

/** Utility, translates tsize into a name. */
function tsize_name($ts=-1)
{
  global $tsize;
  if($ts<0 || $ts>1)
    $ts=$tsize;
  if($ts) return 'album';
  else    return 'thumb';
}

/** Determines whether $fname is the name an image file.
 *  If so, it return its MIME type. */
function image_mime($fname)
{
  $exts=array(
    '%/[^.][^/]*[.]jpe?g$%i'=>'image/jpeg',
    '%/[^.][^/]*[.]png$%i'=>'image/png',
    '%/[^.][^/]*[.]gif$%i'=>'image/gif'
  );
  foreach($exts as $patt => $mime)
  {
    if( preg_match($patt,$fname) && is_file($fname) && is_readable($fname) )
        return $mime;
  }
  return FALSE;
}

function hsc($s)
{
  return htmlspecialchars($s);
}

$admin_message_text='';
function admin_message($msg)
{
  global $admin_message_text;
  $admin_message_text.='<div class="alert">'.$msg.'</div>';
}

/** Takes an EXIF format rational number and translates to a decimal. */
function rational2dec($rational)
{
  $num=explode('/',$rational);
  if(count($num)==1)
    $result=intval($num[0]);
  if(count($num)==2)
    eval('$result='.intval($num[0]).'/'.intval($num[1]).';');
  return strval($result);
}

/** Turns file basenames into 'nice' HTML. */
function nice($s)
{
  return hsc( str_replace('_',' ',$s) );
}

?>
<?php // dirs.php

/** Singleton class responsible for finding relative paths.
 */
class Dirs
{
  var $checked;
  var $path_links;
  var $imagefiles;
  var $directories;
  var $top; ///< path to the top directory. E.g. ../..
  function Dirs()
  {
    $this->checked=array();
    $this->path_links=array(); // { name -> link }
    // Make sure that the _exa directory exists.
    $this->album('.');
    // Find out where we are.
    $path=getcwd();
    $link=array();
    while(TRUE)
    {
      if(!is_dir($path.'/_exa'))
          break;
      $L=implode('/',array_reverse($link));
      $this->top=strlen($L)?$L:'.';
      $this->path_links[hsc(basename($path))]=$this->top;
      $link[]='..';
      $newpath=dirname($path);
      if($newpath==$path)
          break;
      $path=$newpath;
    }
    $this->path_links=array_reverse($this->path_links,true);
    // Read the contents of the current directory.
    list($this->directories,$this->imagefiles)=$this->read_contents();
  }
  /** Ensures that the named directory is exists, and return the path. */
  function get_dir($dirname)
  {
    if(!array_key_exists($dirname,$this->checked))
    {
      if(!is_dir($dirname))
      {
        mkdir($dirname,0777);
        chmod($dirname,0777);
      }
      $this->checked[$dirname]=TRUE;
    }
    return $dirname;
  }
  /** Returns the path of the current album dir. */
  function album($root)
  {
    return $this->get_dir($root.'/_exa');
  }
  /** Returns the relative path of the *top* _exa dir. */
  function top_album()
  {
    return $this->top.'/_exa';
  }
  /** Returns the relative path to the _exa/<image>.d directory. */
  function image($image)
  {
    return $this->get_dir(
      $this->album($image->info['dirname']).'/'.$image->info['basename'].'.d'
    );
  }
  /** Returns the relative path FROM root TO here. */
  function path()
  {
    global $options;
    $result=array_keys($this->path_links);
    $result[0]=$options->top_name;
    return implode('/',$result);
  }
  /** Returns the relative path FROM root TO here, as a series of
   *  HTML links.
   *  Set parameter to
   *   - TRUE:    forces last link
   *   - FALSE:   suppresses last link
   *   - default: last is link if there are sub-directories
   */
  function path_links($link_last=1)
  {
    global $options;
    if($link_last!==TRUE && 0==count($this->directories))
      $link_last=FALSE;
    $result=array_keys($this->path_links);
    $result[0]=$options->top_name;
    $links=array_values($this->path_links);
    $len=count($result);
    for($i=0; $i<$len; $i++)
    {
      // Record where we came from, so that doc_dir() can go to the correct page
      if($i<$len-1)
        $from_dir='/index.php?from_dir='.urlencode($result[1+$i]);
      else
        $from_dir='';

      if($link_last || $i<$len-1)
        $result[$i]='<a href="'.urlencode($links[$i]).$from_dir.'">'
                   .$result[$i].'</a>';
      if($i==$len-1)
        $result[$i]='<strong>'.$result[$i].'</strong>';
    }
    $result=implode('/',$result);
    if(strlen($options->home_url))
    {
      $result=sprintf('<a href="%s">%s</a>&nbsp;&nbsp;'."\n",
                $options->home_url,
                $options->home_name
              ).$result;
    }
    return $result;
  }
  /** Returns a sorted list of images in $dirname. */
  function read_contents($dirname='.')
  {
    $imagefiles=array();
    $directories=array();
    if($handle=opendir($dirname))
    {
      while(FALSE !== ($fname=readdir($handle)))
      {
        $fpath=$dirname.'/'.$fname;
        if(is_dir($fpath) && is_writeable($fpath))
        {
          if(preg_match('/^[^._]/',$fname))
              $directories[]=$fname;
        }
        elseif($mime=image_mime($fpath))
        {
          $im=new ImageFile($fpath,$mime);
          //$imagefiles[$fname]=new ImageFile($dirname.'/'.$fname,$mime);
          $imagefiles[sprintf('%012d_%s',$im->date_time(),$fname)]=$im;
        }
      } // end while
      closedir($handle);
      ksort($imagefiles);
    }
    return array($directories,array_values($imagefiles));
  }
  function current_album_title()
  {
    global $options;
    if(count($this->path_links)>1)
      return nice(basename(getcwd()));
    else
      return $options->top_name;
  }
  
}
// Construct the global singleton.
$dirs=new Dirs();

?>
<?php // options.php

class Options
{
  var $secret;
  var $passwd='';
  var $comments=1; ///< Allows visitors to leave comments.
  var $exif=1; ///< List EXIF data, where available.
  var $reload=0; ///< Always regenerate 'static' files (for development/upgrade)
  var $top_name='gallery';
  var $home_url=''; ///< If set, generates a link on every page.
  var $home_name='home'; ///< Text for home link.
  var $slideshow_delay=4000;  ///< Min delay between slides, ms.
  var $date_format='j M Y, H:i';
  var $comment_wait_min=4;
  var $comment_wait_max=3600;
  var $theme='DEFAULT';

  function Options()
  {
    global $dirs, $is_admin;
    $is_admin=false;

    // Read in configuration
    $config_fname=$dirs->top_album().'/config.php';
    if(file_exists($config_fname))
    {
      include $config_fname;
      // Check admin password.
      $passwd='';
      if(array_key_exists('passwd',$_POST))
      {
        $passwd=md5($_POST['passwd'].$this->secret);
        setcookie('exa_passwd',$passwd,0,'/');  
      }
      elseif(array_key_exists('logout',$_GET))
      {
        setcookie('exa_passwd','',time()-100000,'/'); // Unset cookie
        admin_message('Logged out.');
      }
      elseif(array_key_exists('exa_passwd',$_COOKIE))
      {
        $passwd=$_COOKIE['exa_passwd'];
      }
      if(0==strcmp($passwd,$this->passwd))
        $is_admin=true;
      elseif(array_key_exists('passwd',$_POST))
        admin_message('Incorrect password.');
    }
    else
    {
      // Set secret by picking 20 random characters from a list.
      $this->secret='';
      $possible='0123456789abcdfghjkmnpqrstvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ-_';
      while(strlen($this->secret) < 20)
        $this->secret.=substr($possible, mt_rand(0,strlen($possible)-1), 1);
      $this->exif=function_exists('exif_read_data');
    }
  }
  
  function save()
  {
    global $dirs;
    $config_fname=$dirs->top_album().'/config.php';
    $handle=fopen($config_fname,'wb');
    fwrite($handle,"<?php\n");
    $opts=get_object_vars($this);
    foreach($opts as $k => $v)
        fwrite($handle,sprintf("\$this->%s='%s';\n",$k,$v));
    fwrite($handle,"?>\n");
    fclose($handle);
  }
}
// Construct the global singleton
$options=new Options();

?>
<?php // imageref.php

/** Holds info about a single image. */
class ImageRef
{
  function ImageRef($src,$width=0,$height=0)
  {
    $this->src=$src;
    if($width)
    {
      $this->width=$width;
      $this->height=$height;
    }
    else
    {
      list($this->width,$this->height) = getimagesize($src);
    }
  }
  function html()
  {
    return 'src="'.$this->src.'" '
         . 'width="'.$this->width.'" '
         . 'height="'.$this->height.'"';
  }
  function ehtml()
  {
    echo $this->html();
  }
}

?>
<?php // comment.php

/** A comment, complete with contents. */
class Comment
{
  function Comment($path,$fname=FALSE)
  {
    if($fname===FALSE)
      $fname=basename($path);
    $this->path=$path;
    $this->id=$fname;
    $c=file_get_contents($path);
    list($this->author,$this->text) = explode("\n",$c."\n",2);
    list($t,$dummy) = explode('-',$fname.'-',2);
    $this->time=intval($t);
  }
  function square()
  {
    return dirname(dirname($this->path)).'/square.jpg';
  }
  function link()
  {
    $imgdir=dirname(dirname($this->path));
    $fname=basename($imgdir);
    $fname=substr($fname,0,strlen($fname)-2);
    $dirname=dirname(dirname($imgdir));
    if('.'==$dirname)
      return '?d=c&f='.urlencode($fname);
    else
      return $dirname.'/?d=c&f='.urlencode($fname);
  }
}

?>
<?php // imagefile.php

/** Representation of an image. */
class ImageFile
{
  var $fname;
  var $info;
  var $mime;
  var $name;
  var $exif='';
  var $exif_caption='';
  var $dt=-1; // date time (unixtime)
  function ImageFile($fname,$mime)
  {
    $this->mime=$mime;
    $this->init_fname($fname);
    $this->read_exif();
  }
  
  function init_fname($fname)
  {
    $this->fname=$fname;
    $this->info=pathinfo($fname);
    $b=$this->info['basename'];
    $e=$this->info['extension'];
    $this->name=substr($b,0,strlen($b)-strlen($e)-1);
  }
  
  function move($fname)
  {
    global $dirs;
    if(file_exists($fname))
        return FALSE;
    $old_thumb_dir=$dirs->image($this);
    if( rename($this->fname,$fname) )
    {
      $this->init_fname($fname);
      $new_thumb_dir=$dirs->image($this);
      return rename($old_thumb_dir,$new_thumb_dir);
    }
    return FALSE;
  }
  
  function set_title($new_title)
  {
    return $this->move(
      $this->info['dirname'].'/'
      .preg_replace('%[ \\/]%','_',$new_title)
      .'.'.$this->info['extension']
    );
  }
  
  /** Loads the image */
  function image()
  {
    if($this->mime=='image/jpeg')
        return imagecreatefromjpeg($this->fname);
    else if($this->mime=='image/png')
        return imagecreatefrompng($this->fname);
    else if($this->mime=='image/gif')
        return imagecreatefromgif($this->fname);
    else
        return FALSE;
  }
  
  /** Returns the square filename. */
  function square()
  {
    global $dirs;
    $result=$dirs->image($this).'/square.jpg';
    if(!file_exists($result) || filemtime($result)<filemtime($this->fname))
    {
      // Make a small square thumbnail.
      list($w0,$h0) = getimagesize($this->fname);
      if($w0<$h0)
      { // Tall
        $d0=$w0;
        $x0=0;
        $y0=($h0-$w0)/2;
      }
      else
      { // Wide
        $d0=$h0;
        $x0=($w0-$h0)/2;
        $y0=0;
      }
      // Zoom in a bit more.
      $fringe=$d0/10;
      $x0+=$fringe;
      $y0+=$fringe;
      $d0-=(2*$fringe);

      $image=$this->image();
      $square=imagecreatetruecolor(75,75);
      imagecopyresampled(
        $square,
        $image,
        0,0,   // dst x,y
        $x0,$y0, // src x,y
        75,75,
        $d0,$d0
      );
      // Save it.
      imagejpeg($square,$result,85);
      imagedestroy($square);
      chmod($result,0666);
    }
    return new ImageRef($result,75,75);
  }

  /** Returns the resized filename. */
  function resized($img_size)
  {
    global $dirs;
    
    if($img_size==0)
      return $this->square();
    elseif($img_size==1)
    {
      $name='small';
      $W=240;
      $H=240;
    }
    elseif($img_size==2)
    {
      $name='carousel';
      $W=500;
      $H=500;
    }
    elseif($img_size==3)
    {
      $name='medium';
      $W=800;
      $H=533;
    }
    elseif($img_size==4)
    {
      $name='large';
      $W=1024;
      $H=700;
    }
    else
    {
      return new ImageRef($this->fname);
    }
    
    $result=$dirs->image($this).'/'.$name.'.jpg';
    if(!file_exists($result) || filemtime($result)<filemtime($this->fname))
    {
      // Make a file.
      list($w0,$h0) = getimagesize($this->fname);
      if( $h0*$W/$w0 <= $H)
      {
        $w1=$W;
        $h1=$h0*$W/$w0;
      }
      else
      {
        $w1=$w0*$H/$h0;
        $h1=$H;
      }
      $image=$this->image();
      $resized=imagecreatetruecolor($w1,$h1);
      imagecopyresampled(
        $resized,
        $image,
        0,0, // dst x,y
        0,0, // src x,y
        $w1,$h1,
        $w0,$h0
      );
      // Save it.
      imagejpeg($resized,$result,85);
      imagedestroy($resized);
      chmod($result,0777);
    }
    return new ImageRef($result,$w1,$h1);
  }

  /** Reads the EXIF data for this image. */
  function read_exif()
  {
    if(function_exists('exif_read_data'))
    {
      // exif_read_data() can spew warnings, so suppress them with @
      $exif_data= @ exif_read_data($this->fname,'EXIF');
      if($exif_data===FALSE)
          return;
      
      // Create the summary string.
      $pieces=array();
      $fields=array('Model'=>'<strong>%s</strong>',
        'FocalLength'=>'%smm', // 'DateTime'=>'%s',
        'ExposureTime'=>'%ss', 'FNumber'=>'f%s', 'ISOSpeedRatings'=>'ISO%d');
      $rationals=array('FNumber','FocalLength');
      foreach($fields as $k => $templ)
      {
        if(array_key_exists($k,$exif_data))
        {
          $v=$exif_data[$k];
          if(in_array($k,$rationals))
            $v=rational2dec($v);
          $pieces[]=sprintf($templ,$v);
        }
      }
      $this->exif=implode(",\n",$pieces);

      // Read the date time
      if(array_key_exists('DateTime',$exif_data))
      {
        $date_time=explode(' ',$exif_data['DateTime']);
        if(count($date_time)>=2)
        {
          $d=explode(':',$date_time[0]);
          $t=explode(':',$date_time[1]);
          if(count($d)>=3 && count($t)>=3)
              $this->dt=mktime(
                 intval($t[0]),intval($t[1]),intval($t[2]),
                 intval($d[1]),intval($d[2]),intval($d[0])
              );
        }
      }
      
      // Read the EXIF comment
      if(array_key_exists('COMPUTED',$exif_data))
      {
        $c=$exif_data['COMPUTED'];
        if(isset($c['UserComment']) && isset($c['UserCommentEncoding']))
        {
          if('ASCII'==$c['UserCommentEncoding'])
              $this->exif_caption=hsc($c['UserComment']);
          // TODO: support Unicode & JIS encoding here.
        }
      }
    }
  }

  /** Returns the date and time (unix time) of the image, from Exif if
   *  possible, otherwise just the file timestamp. */
  function date_time()
  {
    if($this->dt==-1)
       $this->dt=filemtime($this->fname);
    return $this->dt;
  }

  /** Returns the caption for this image, or FALSE if there is none. */
  function caption()
  {
    global $dirs;
    $caption_fname=$dirs->image($this).'/caption.html';
    $result=false;
    if(file_exists($caption_fname))
    {
      $handle=fopen($caption_fname,'r');
      $caption=fread($handle,1024);
      if(strlen($caption)>0)
        $result=$caption;
      fclose($handle);
    }
    elseif(strlen($this->exif_caption))
    {
      $result=$this->exif_caption;
    }
    return $caption;
  }

  function short_caption($limit=40,$format='%s')
  {
    $caption=$this->caption();
    if(!$caption)
      return '';
    $result=explode("\n",$caption);
    $result=$result[0];
    if(strlen($result)>$limit)
    {
      $words=preg_split('/[\s]+/',$result);
      $c=0;
      for($i=0; $i<count($words) && $c<$limit; $i++)
        $c+=1+strlen($words[$i]);
      $result=implode(' ',array_slice($words,0,$i));
      if(strlen($result) < strlen($caption))
        $result.='...';
    }
    return sprintf($format,$result);
  }
  
  function setcaption($txt)
  {
    global $dirs;
    $caption_fname=$dirs->image($this).'/caption.html';
    $handle=fopen($caption_fname,'w');
    fwrite($handle,$txt);
    fclose($handle);
  }

  function numcomments($format='%s')
  {
    global $dirs;
    $result=0;
    $comments_dir=$dirs->image($this).'/comments';
    if(is_dir($comments_dir) && $dhandle=opendir($comments_dir))
    {
      while(FALSE !== ($fname=readdir($dhandle)))
        if($fname!='.' && $fname!='..')
          $result++;
      closedir($dhandle);
    }
    if($result==1)
      return sprintf($format,$result.'&nbsp;comment');
    elseif($result>1)
      return sprintf($format,$result.'&nbsp;comments');
    else
      return '';
  }

  function comments()
  {
    global $dirs;
    $result=array();
    $comments_dir=$dirs->image($this).'/comments';
    if(is_dir($comments_dir) && $dhandle=opendir($comments_dir))
    {
      while(FALSE !== ($fname=readdir($dhandle)))
      {
        $path=$comments_dir.'/'.$fname;
        if(is_file($path) && is_readable($path))
        {
          $result[$fname]=new Comment($path,$fname);
        }
      } // end while
      closedir($dhandle);
      if(count($result))
      {
        ksort($result);
        return $result;
      }
    }
    return FALSE;
  }

  function addcomment($author,$comment)
  {
    global $dirs;
    $comments_dir=$dirs->get_dir( $dirs->image($this).'/comments' );
    // If the same IP address submits two comments in the same second, then
    // we lose one. Ho hum.
    $fname=sprintf($comments_dir.'/%012d-%s',time(),$_SERVER['REMOTE_ADDR']);
    $handle=fopen($fname,"w");
    if($handle!==FALSE)
    {
      fwrite($handle,$author."\n".$comment);
      fclose($handle);
    }
  }

  function delcomment($cid)
  {
    global $dirs;
    $comments_dir=$dirs->get_dir( $dirs->image($this).'/comments' );
    $fname=$comments_dir.'/'.$cid;
    if(is_file($fname))
        unlink($fname);
  }

}

?>
<?php // paged.php

/** Implements multi-page documents. */
class Paged
{
  function Paged($items_per_page,$num_items,$current_item=0)
  {
    if(array_key_exists('p',$_GET))
      $p=intval($_GET['p']); // page
    else
      $p=intval($current_item/$items_per_page);
    $this->items_per_page=$items_per_page;
    $this->num_items=$num_items;
    $this->num_pages=ceil($num_items/$items_per_page);
    $this->p=minmax(0,$this->num_pages-1,$p);
  }
   /** Starting point for the current page. */
  function begin()
  {
    return( $this->items_per_page * $this->p );
  }
  /** End point for the current page. */
  function end()
  {
    return min( $this->num_items, $this->items_per_page * (1+$this->p) );
  }
  /** Emit a page index. Prev/Next buttons, highlights current page. */
  function index($link)
  {
    $bracket=5;
    $run=$bracket*2+1;
    $spacer=False;
    echo '<div id="pages">';
    if($this->p>0)
      echo '<a href="'.$link.'p='.($this->p-1).'"> &nbsp; &lt; &nbsp;</a>';
    else
      echo '<span class="off"> &nbsp; &lt; &nbsp;</span>';
    for($j=0; $j<$this->num_pages; $j++)
    {
      if($this->p==$j)
      {
        echo '<span class="here">'.($j+1).'</span>';
        $spacer=False;
      }
      elseif($j==0 || $j+1==$this->num_pages || abs($j-$this->p)<$bracket
        || ($this->p<$bracket && $j+1<$run)
        || ($this->p+$bracket>=$this->num_pages && $j+$run-1>=$this->num_pages) )
      {
        echo '<a href="'.$link.'p='.$j.'">'.($j+1).'</a>';
        $spacer=False;
      }
      elseif(!$spacer)
      {
        echo '...';
        $spacer=True;
      }
    }
    if($this->p+1<$this->num_pages)
      echo '<a href="'.$link.'p='.($this->p+1).'">&nbsp; &gt; &nbsp; </a>';
    else
      echo '<span class="off">&nbsp; &gt; &nbsp; </span>';
    echo '</div>';
  }
}

?>
<?php // common.php

// Functions that generate HTML common across several document types.

/** Checks that the $img exists, otherwise returns HTTP 404. */
function assert_img()
{
  global $dirs,$img;
  $c=count($dirs->imagefiles);
  if(0==$c || $img<0 || $img>=$c)
  {
    die("404"); // ?? Fix this to generate a proper error page.
  }
}

/** Emits the set of switcher icons in the #path bar at the top of the screen.
 */
function switcher($doc)
{
  global $img;
  if($img)
    $img_param='&amp;img='.$img;
  $i='<img title="%1$s" alt="%1$s" src="%2$s" width="16" height="16" />';

  echo "&nbsp;&nbsp;\n";

  if($doc=='t')
    $a='<span>%s</span>';
  else
    $a='<a id="nav_t" href="?d=t'.$img_param.'">%s</a>';
  printf($a,sprintf( $i, 'album', get_file(tsize_name().'.gif') ));

  if($doc=='c')
    $a='<span>%s</span>';
  else
    $a='<a id="nav_c" href="?d=c'.$img_param.'">%s</a>';
  printf($a,sprintf( $i, 'carousel', get_file('browse.gif') ));

  if($doc=='s')
    $a='<span>%s</span>';
  else
    $a='<a id="nav_s" href="?d=s'.$img_param.'">%s</a>';
  printf($a,sprintf( $i, 'single', get_file('single.gif') ));
}

/** Emits a small status box, that allows the admin to log in and out, and
 *  displays the Easy Ajax Album logo. */
function status()
{
  global $is_admin,$options;
?>

<div id="status">
 powered by <a title="version <?php
 echo EXA_VERSION ?>" href="<?php
 echo EXA_URL ?>">Easy Ajax Album</a> |
<?php if(intval($options->comments)): ?>
 <a href="?d=l">comments</a> |
<?php endif ?>

<?php if($is_admin): ?>
 <a href="?d=a">prefs</a> |
 <a href="?logout">logout</a>
<?php else: ?>
 <a href="?d=a">login</a>
<?php endif ?>
</div>

<?php
}

function img_load($id)
{
  echo '<img id="'.$id.'" src="'.get_file('load.gif')
    .  '" width="16" height="16" alt="loading" style="display:none" />'."\n";
}

/** Loads a stylesheet called theme.css, if it exists. */
function load_theme()
{
  global $dirs,$options;
  if($options->theme=='DEFAULT')
      return;
  $theme=$dirs->top_album().'/themes/'.$options->theme.'.css';
  if(file_exists($theme))
      echo '<link rel="stylesheet" href="'.$theme.'" type="text/css" />'."\n";
}

?>
<?php // doc_xml.php

function doc_xml(&$imagefiles)
{
  global $dirs,$img,$isize,$options;
  header('Expires: Mon, 26 Jul 1997 05:00:00 GMT'); // disable IE caching
  header('Last-Modified: '.gmdate('D, d M Y H:i:s').' GMT'); 
  header('Cache-Control: no-cache, must-revalidate'); 
  header('Pragma: no-cache');
  header('Content-type: text/xml');
  echo '<?xml version="1.0" encoding="UTF-8"?>';
  $im=$imagefiles[$img];
?>
<album length="<?php echo count($imagefiles) ?>">

<image idx="<?php echo $img
   ?>" title="<?php echo nice($im->name)
   ?>" path="<?php echo hsc($dirs->path())
   ?>" <?php $ir=$im->resized($isize); $ir->ehtml()
   ?> href="<?php echo hsc($im->fname)
   ?>">
 <caption><![CDATA[<?php echo $im->caption() ?>]]></caption>
 <brief><![CDATA[<?php echo $im->short_caption() ?>]]></brief>
<?php
 global $admin_message_text;
 if(strlen($admin_message_text)>0)
    echo '<alert>'.$admin_message_text.'</alert>';
 $comments=$im->comments();
 if($comments)
    foreach($comments as $c): ?>
      <comment id="<?php echo $c->id ?>">
       <text><?php echo $c->text ?></text>
       <author><?php echo $c->author ?></author>
       <date><?php echo date($options->date_format,$c->time) ?></date>
      </comment>
<?php endforeach; ?>
</image>

</album>
<?php }

?>
<?php // doc_admin.php

function div_setpasswd($focus=true)
{
?>

<div id="header">
 <h1>Set Your Admin Password</h1>
</div>

<form id="main" name="setpasswd" action="?d=a" method="POST">
 <input type="hidden" id="do" name="do" value="setpasswd">
 <table><tbody>
  <tr>
   <td><label for="p1">Password:</label></td>
   <td><input id="p1" name="p1" type="password" /></td>
  </tr>
  <tr>
   <td><label for="p2">...and again:</label></td>
   <td><input id="p2" name="p2" type="password" /></td>
  </tr>
  <tr>
   <td colspan="2" style="text-align:right"><input type="submit" value="Set Password" /></td>
  </tr>
 </tbody></table>
</form>

<?php if($focus): ?>

<script type="text/javascript"><!--
 window.onload=function(){document.forms['setpasswd'].p1.focus();}
// --></script>

<?php endif;
}


function div_login()
{
?>

<div id="header">
 <h1>Admin Login</h1>
</div>

<form id="main" name="login" action="?d=a" method="POST">
 <table><tbody>
  <tr>
   <td><label for="passwd">Password:</label></td>
   <td><input id="passwd" name="passwd" type="password" /></td>
  </tr>
  <tr>
   <td colspan="2" style="text-align:right"><input type="submit" value="Log In" /></td>
  </tr>
 </tbody></table>
</form>

<script type="text/javascript"><!--
 window.onload=function(){document.forms['login'].passwd.focus();}
// --></script>

<?php
}


function checked($v)
{
  if(intval($v))
    echo ' checked="checked"';
}


function value($v,$current)
{
  echo ' value="'.$v.'"';
  if($v==$current)
    echo ' selected="selected"';
}


function select_themes($id)
{
  global $dirs,$options;

  // Work out where the themes directory is, and make sure that all the
  // built-ins are unpacked.
  $themesdir=$dirs->get_dir( $dirs->top_album().'/themes' );
  foreach(array('black','neutral') as $t)
      get_file('themes/'.$t.'.css');
  ?>

    <select id="<?php echo $id ?>" name="<?php echo $id ?>">
     <option<?php value('DEFAULT',$options->theme) ?>>White (default)</option>
<?php 
     $themes=array();
     if($handle=opendir($themesdir))
     {
       while(FALSE !== ($fname=readdir($handle)))
       {
         if(preg_match('/^[^._].*[.]css$/',$fname))
             $themes[]=$fname;
       } // end while
       closedir($handle);
       sort($themes);
     }
     foreach($themes as $t)
     {
       $name=preg_replace('/[.]css$/','',$t);
       echo '     <option';
       value($name,$options->theme);
       echo '>'.$name.'</option>'."\n";
     } ?>
    </select><?php
}


function div_prefs()
{
  global $options;

?>

<div id="header">
 <h1>Preferences</h1>
</div>

<div id="main">

<form action="?d=a" method="POST">
 <input type="hidden" id="do" name="do" value="setprefs">
 <table><tbody>
  <tr>
   <td><label for="top_name">Gallery Name:</label></td>
   <td><input id="top_name" name="top_name" type="text" value="<?php
    echo $options->top_name ?>" /></td>
  </tr>
  <tr>
   <td><label for="home_name">Home:</label></td>
   <td><input id="home_name" name="home_name" type="text" value="<?php
    echo $options->home_name ?>" /></td>
  </tr>
  <tr>
   <td><label for="home_url">Home URL:</label></td>
   <td><input id="home_url" name="home_url" type="text" value="<?php
    echo $options->home_url ?>" /></td>
  </tr>
  <tr>
   <td><label for="theme">Theme:</label></td>
   <td><?php select_themes('theme') ?></td>
  </tr>
  <tr>
   <td><label for="slideshow_delay">Slide show speed:</label></td>
   <td><input id="slideshow_delay" name="slideshow_delay" type="text" value="<?php
    echo intval($options->slideshow_delay)/1000 ?>" size="3" style="text-align:right" />
    <em>seconds</em></td>
  </tr>
  <tr>
   <td><label for="comments">Allow Comments:</label></td>
   <td><input id="comments" name="comments" type="checkbox" value="1"<?php
    checked($options->comments) ?>" /></td>
  </tr>
  <tr>
   <td><label for="exif">Show EXIF data:</label></td>
   <td><input id="exif" name="exif" type="checkbox" value="1"<?php
    checked($options->exif) ?>" /></td>
  </tr>
  <tr>
   <td><label for="reload">Regenerate files (test mode):</label></td>
   <td><input id="reload" name="reload" type="checkbox" value="1"<?php
    checked($options->reload) ?>" /></td>
  </tr>
  <tr>
   <td colspan="2" style="text-align:right"><input type="submit" value="Save Preferences" /></td>
  </tr>
 </tbody></table>
</form>

</div>

<?php
}


function doc_admin()
{
  global $dirs,$options,$is_admin;

?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>

<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
<title>Easy Ajax Album Administration</title>
<link rel="stylesheet" href="<?php echo get_file('exa.css') ?>" type="text/css" />
<?php load_theme() ?>
</head>

<body>

<div id="wrap" class="admin">

<div id="path" class="iconbox">
<?php echo $dirs->path_links() ?>
<?php switcher('a') ?>
</div>

<?php

  global $admin_message_text;
  echo $admin_message_text;

  if(0==strlen($options->passwd)):
      div_setpasswd();
  elseif($is_admin):
      div_prefs();
      div_setpasswd(false);
  else:
      div_login();
  endif;

  status();

?>

</div> <!-- wrap -->

</body>
</html>
<?php } // doc_admin() ---------------------------------------------------------

?>
<?php // doc_single.php

function doc_single(&$imagefiles)
{
  assert_img();
  global $dirs,$img,$isize,$options;

  $len=count($imagefiles);

  $title=nice($imagefiles[$img]->name);
  $album=basename(getcwd());

  $prev=normalize($img+$len-1,$len);
  $next=normalize($img+$len+1,$len);

  $on_tmpl='class="on" href="?d=s&amp;img=%s&amp;isize=%s"';
  $off='class="off" href="#"';
  if($isize>0)
    $smaller=sprintf($on_tmpl,$img,$isize-1);
  else
    $smaller=$off;

  if($isize<5)
    $bigger=sprintf($on_tmpl,$img,$isize+1);
  else
    $bigger=$off;

?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>

<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
<title><?php echo $title.' &laquo; '.hsc($dirs->path()) ?></title>
<link rel="stylesheet" href="<?php echo get_file('exa.css') ?>" type="text/css" />
<?php load_theme() ?>
<script type="text/javascript" src="<?php echo get_file('xmlhttprequest.js') ?>"></script>
<script type="text/javascript" src="<?php echo get_file('util.js') ?>"></script>
<script type="text/javascript" src="<?php echo get_file('slideshow.js') ?>"></script>
<script type="text/javascript"><!--
 var ss=new SlideShow(<?php
   echo $img.','.$len.','.$isize.','.intval($options->slideshow_delay) ?>);
<?php if($_GET['ss']) echo ' window.onload=function(){ss.do_play();}' ?>
//--></script>
<!-- Additional IE/Win specific style sheet (Conditional Comments) --><!--[if IE]>
<style type="text/css" media="projection, screen">
  /* use Star Selector Hack here to exclude IE 7 - with the Conditional Comment [if lte IE 6] it didn't work, WTF?! */
  * html img.fixpng {
    background-image: expression(
     this.runtimeStyle.filter = 'progid:DXImageTransform.Microsoft.AlphaImageLoader(src="' + this.src + '", sizingMethod="image")',
     this.runtimeStyle.backgroundImage = 'none',
     this.src = '<?php echo get_file('transparent.gif') ?>'
    );
  }
</style><![endif]-->
</head>

<body>

<div id="wrap">

<div id="path" class="iconbox">
<a id="download" href="?dl&amp;img=<?php echo $img ?>">download</a>
<?php echo $dirs->path_links() ?>
<?php switcher('s') ?>
</div>

<table id="header"><tbody><tr><td>
  <h1 id="title"><?php echo $title ?></h1>
</td><td class="bottom_right iconbox">
  <?php echo img_load('busy') ?>

  <a class="on" id="prev" href="?d=s&amp;img=<?php echo $prev
  ?>" onclick="javascript:ss.do_prev();return false">
    <img title="previous image" class="fixpng" src="<?php echo get_file('rew.png')
    ?>" width="16" height="16" alt="rew" /></a>

  <a class="on" id="play" href="javascript:ss.do_play()">
    <img title="play slide show" class="fixpng" src="<?php echo get_file('play.png')
    ?>" width="16" height="16" alt="play" /></a>

  <a class="on" id="stop" href="javascript:ss.do_stop()" style="display:none">
    <img title="stop slide show" src="<?php echo get_file('stop.gif')
    ?>" width="16" height="16" alt="stop" /></a>

  <a class="on" id="next" href="?d=s&amp;img=<?php echo $next
  ?>" onclick="javascript:ss.do_next();return false">
    <img title="next image" class="fixpng" src="<?php echo get_file('fwd.png')
    ?>" width="16" height="16" alt="fwd" /></a>

  &nbsp;&nbsp;

  <a <?php echo $smaller ?> id="smaller" onclick="javascript:ss.do_smaller();return false">
    <img title="smaller" src="<?php echo get_file('smaller.gif')
    ?>" width="16" height="16" alt="smaller" /></a>

  <a <?php echo $bigger ?> id="bigger" onclick="javascript:ss.do_bigger();return false">
    <img title="bigger" src="<?php echo get_file('bigger.gif')
    ?>" width="16" height="16" alt="bigger" /></a>

</td></tr></tbody></table>

</div> <!-- wrap -->

<div id="single">
<img id="img_<?php echo $img.'_'.$isize
     ?>" <?php $ir=$imagefiles[$img]->resized($isize); $ir->ehtml() ?> />
</div>

<?php status() ?>

</body>
</html>
<?php } // doc_single() --------------------------------------------------------

?>
<?php // doc_thumb.php

function doc_thumb(&$imagefiles)
{
  global $dirs,$tsize,$img;

  $len=count($imagefiles);

?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>

<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
<title>thumbnails &laquo; <?php echo hsc($dirs->path()) ?></title>
<link rel="stylesheet" href="<?php echo get_file('exa.css') ?>" type="text/css" />
<?php load_theme() ?>
</head>

<body>

<div id="wrap" class="thumbs">

<div id="path" class="iconbox">
<?php echo $dirs->path_links() ?>
<?php switcher('t') ?>
</div>

<table id="header"><tbody><tr><td>
  <h1><?php echo $dirs->current_album_title() ?></h1>
</td><td class="bottom_right iconbox">
<?php

$t=array('thumb','album');
if($img)
  $img_param='&amp;img='.$img;
for($i=0; $i<2; $i++)
{
  $ts=tsize_name($i);
  if($i==$tsize)
    $a='<span>%s</span>';
  else
    $a='<a id="'.$ts.'" href="?d=t&amp;tsize='.$i.$img_param.'">%s</a>';
  printf($a,'<img title="'.$ts.'" src="'.get_file($ts.'.gif')
             .'" width="16" height="16" alt="'.$ts.'" />');
}

?>
</td></tr></tbody></table>

<div id="main">

<?php if($tsize==0): $paged=new Paged(42,$len,$img) ?>

  <div id="thumbs">
  <?php
  for($i=$paged->begin(); $i<$paged->end(); $i++)
  {
    $image=$imagefiles[$i];
    $ir=$image->resized(0);
    $alt=nice($image->name);
    $title=$alt;
    $caption=$image->short_caption();
    if($caption)
      $title.=': '.$caption;
    echo "\n";
    echo '  <a title="'.$title.'" href="?d=c&amp;img='.$i.'">'."\n"
    .    '  <img alt="'.$alt.'" '.$ir->html()." />\n"
    .    "  </a>";
  }
  echo "\n";
  $paged->index('?d=t&amp;')
  ?>
  </div> <!-- thumbs -->

<?php else: $paged=new Paged(18,$len,$img) ?>

  <table id="thumbs"><tbody><tr>
  <?php
    $rows=array();
    for($i=$paged->begin(); $i<$paged->end(); $i++)
    {
      if(0==($i % 3))
        $rows[]='';
      $ir=$imagefiles[$i]->resized(1);
      $rows[count($rows)-1].=
        '<td><h3>'.nice($imagefiles[$i]->name).'</h3>'
       .'<a title="'.nice($imagefiles[$i]->name)
       .'" href="?d=c&amp;img='.$i.'"><img '.$ir->html().' /></a>'
       .$imagefiles[$i]->short_caption(100,'<p class="caption">%s</p>')
       .$imagefiles[$i]->numcomments('<p class="num_comments">%s</p>')."</td>\n  ";
    }
    echo implode("</tr>\n  <tr>",$rows);
  ?>
  </tr></tbody></table>
  <?php $paged->index('?d=t&amp;') ?>

<?php endif ?>

</div> <!-- main -->

<?php status() ?>

</div> <!-- wrap -->

</body>
</html>
<?php } // doc_thumb() ---------------------------------------------------------

?>
<?php // doc_carousel.php

function h1_title($title,$img)
{
  global $is_admin;
  if($is_admin): ?>
  
    <h1 id="title_block">
     <span id="title"><?php echo $title ?></span>
     <a id="title_ctrl" class="control" onclick="return do_show_title_editor()" href="#">edit</a>
    </h1>
    
    <form id="title_form" class="title" style="display:none" name="title_form" method="POST" action="?d=x&img=<?php echo $img ?>">
     <input type="hidden" name="do" value="edittitle" />
     <input type="text" id="title_txt" name="title_txt" value="<?php echo $title ?>" />
     <input type="submit" id="title_sub" value="Save" title="Save Title" onclick="return do_save_title()" />
     <?php echo img_load('title_busy') ?>
    </form>

  <?php else: ?>

    <h1><?php echo $title ?></h1>

  <?php endif;
}

function div_carousel_thumb($i,&$imagefiles,$curr,$display=TRUE)
{
  global $options;
  $len=count($imagefiles);
  $im=$imagefiles[$i];

  $dt=date($options->date_format,$im->date_time());
  $nc=$im->numcomments('<span id="nc_'.$i.'" class="num_comments">%s</span>');
  ?>

    <div id="img_<?php echo $i;
         ?>" class="thumb<?php if($i==$curr) echo ' here';
                                if($i==$len-1) echo ' img_last';
         ?>"<?php if(!$display) echo ' style="display:none"';
         ?>>
     <a onclick="return go(this)" href="<?php
       echo '?d=c&amp;img=' . $i
       ?>"><img <?php $ir=$im->square(); $ir->ehtml() ?> /></a>
     <div>
      <h3 id="ttl_<?php echo $i ?>"><?php echo nice($im->name) ?></h3>
      <p id="dtc_<?php echo $i ?>"><?php echo $nc ?><span class="date_time"><?php echo $dt ?></span></p>
      <p id="cap_<?php echo $i ?>"><?php echo $im->short_caption() ?></p>
     </div>
    </div>

  <?php
  flush();
}


/** Emits a <P> containing the caption for this image, if it has one.
 *  The administrator also sees the controls to add of edit the caption. */
function para_caption(&$imagefile,$img)
{
  global $is_admin;
  $caption=$imagefile->caption();
  if($is_admin): ?>
  
    <p id="cap_para" class="caption">
     <span id="caption"><?php echo str_replace("\n",'<br />',$caption) ?></span>
     <a id="cap_ctrl" class="control" onclick="return do_show_caption_editor()"
      href="?d=c&amp;img=<?php echo $img ?>">
      <?php echo ($caption)? 'edit': 'add caption' ?>
     </a>
    </p>
    
    <form id="cap_form" class="caption" style="display:none" name="cap_form" method="POST" action="?d=x&img=<?php echo $img ?>">
     <input type="hidden" name="do" value="editcaption" />
     <textarea id="cap_txt" name="cap_txt"><?php echo $caption ?></textarea>
     <input type="submit" id="cap_sub" value="Save" title="Save Caption" onclick="return do_save_caption()" />
     <?php echo img_load('cap_busy') ?>
    </form>
  
  <?php elseif($caption): ?>

    <p class="caption"><?php echo $caption ?></p>

  <?php endif;
}


function div_comments(&$imagefile)
{
  global $img,$options,$is_admin;
  $comments=$imagefile->comments();
  echo '<div id="comments">'."\n";
  if($comments)
  {
    echo '<h2 id="cmt_head">Comments</h2>'."\n";
    foreach($comments as $c)
    {
      global $options;
      echo '<div class="comment" id="'.$c->id.'">'."\n";
      echo '<p>'.$c->text."</p>\n";
      echo '<p class="by">posted by <span class="author">'.$c->author
          .'</span> on '.date($options->date_format,$c->time);
      if($is_admin)
        echo ' <a class="control" onclick="return do_delete_comment(this)" href=".">delete</a>';
      echo '</p></div>'."\n";
    }
  }
  else
  {
    echo '<h2 id="cmt_head" style="display:none">Comments</h2>'."\n";
  }
  
  // Record the time at which the form was generated. Only permit
  // comments to be submitted within a short time, in order to stop spam.
  $timestamp=time();
  $checksum=md5($options->secret.strval($timestamp));
  
  ?>

  <form id="cmt_form" class="comment" name="cmt_form" style="display:none" method="POST" action="?d=x&img=<?php echo $img ?>">
   <h2>Leave a Comment</h2>
   <input type="hidden" name="do" value="addcomment" />
   <input type="text" id="cmt_author" name="cmt_author" />
   <label for="cmt_author">Name</label><br />
   <textarea id="cmt_txt" name="cmt_txt"></textarea><br />
   <input type="submit" id="cmt_sub" value="Submit" title="Submit Comment" onclick="return do_submit_comment()" />
   <?php echo img_load('cmt_busy') ?>
   <span style="display:none">
    <input type="text" id="cmt_dummy" name="url" />
    <label for="cmt_dummy">Dummy anti-spam field. Please leave empty.</label>
    <input type="hidden" name="cmt_ts" value="<?php echo $timestamp ?>" />
    <input type="hidden" name="cmt_chk" value="<?php echo $checksum ?>" />
   </span>
  </form>
  <p class="comment"><!--
   --><a id="cmt_ctrl" class="control" onclick="return do_show_comment_editor()" href="?d=c&amp;img=<?php echo $img
    ?>">Leave a Comment</a></p>

  <?php echo "</div>\n";
}


function doc_carousel(&$imagefiles)
{
  global $dirs,$img,$options;

  $len=count($imagefiles);
  $pos=intval($_POST['pos']);

  $carousel_size=min($len,6); // Number of images in the carousel.

  if(array_key_exists('pos',$_POST))
  {
    // Scroll the carousel around with the image.
    if($pos==$img)
       $pos=$img-1;
    else if(normalize($pos+$carousel_size-1,$len) == $img)
       $pos=$img+2-$carousel_size;
  }
  else
  {
    // Centre the image in the carousel...
    $pos=$img-intval(($carousel_size-0.5)/2);
    // ...but start at the beginning, end at the end.
    if($pos+$carousel_size-1 >= $len)
       $pos=$len-$carousel_size;
    else if($pos < 0)
       $pos=0;
  }
  $pos=normalize($pos,$len);

  $title=nice($imagefiles[$img]->name);
  $album=$dirs->current_album_title();

?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>

<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
<title><?php echo $title.' &laquo; '.hsc($dirs->path()) ?></title>
<link rel="stylesheet" href="<?php echo get_file('exa.css') ?>" type="text/css" />
<?php load_theme() ?>
<script type="text/javascript" src="<?php echo get_file('xmlhttprequest.js') ?>"></script>
<script type="text/javascript" src="<?php echo get_file('util.js') ?>"></script>
<script type="text/javascript" src="<?php echo get_file('carousel.js') ?>"></script>
<script type="text/javascript">
 var img=<?php echo $img ?>;
 var pos=<?php echo $pos ?>;
 var carousel_size=<?php echo $carousel_size ?>;
 var len=<?php echo $len ?>;
</script>
</head>

<body>

<div id="wrap" class="carousel">

<div id="path" class="iconbox">
<a id="download" href="?dl&amp;img=<?php echo $img ?>">download</a>
<?php echo $dirs->path_links() ?>
<?php switcher('c') ?>
</div>

<table id="header"><tbody><tr><td>
  <?php h1_title($title,$img) ?>
</td><td class="bottom_right">
  <h2><?php echo $album ?></h2>
</td></tr></tbody></table>

<div id="main">
<a href="<?php echo '?d=s&amp;img='.$img ?>">
<img <?php $ir=$imagefiles[$img]->resized(2); $ir->ehtml() ?> />
</a>

<?php

para_caption($imagefiles[$img],$img);
if(intval($options->exif) && strlen($imagefiles[$img]->exif))
    echo '<p class="exif">'.$imagefiles[$img]->exif."</p>\n";
if(intval($options->comments))
    div_comments($imagefiles[$img]);

?>

</div>

<div id="browser">

<?php if($len>$carousel_size): ?>
    <form style="display:none" name="carousel" method="POST">
    <input type="hidden" name="pos" value="<?php echo $pos?>" />
    </form>
    <button id="more_up" onmousedown="carousel_up(this)" onmouseup="scr_off()">more</button>
<?php endif;

for($i=0; $i<$carousel_size; $i++)
{
  div_carousel_thumb( ($pos+$i) % $len, $imagefiles,$img );
}

if($len>$carousel_size): ?>
    <button id="more_down" onmousedown="carousel_down(this)" onmouseup="scr_off()">more</button>
    <?php
    for($i=1; $i<$len; $i++)
    {
      if($pos-$i >= 0)
          div_carousel_thumb($pos-$i,$imagefiles,$img,FALSE);
      if($pos+$carousel_size+$i-1 < $len)
          div_carousel_thumb($pos+$carousel_size+$i-1,$imagefiles,$img,FALSE);
    }
endif;

?>

</div> <!-- carousel -->

<?php status() ?>

</div> <!-- wrap -->

</body>
</html>
<?php } // doc_carousel() ------------------------------------------------------

?>
<?php // doc_dir.php

class Album
{
  var $dirname;
  var $directories;
  var $imagefiles;

  function Album($dirname='.')
  {
    global $dirs;
    $this->dirname=$dirname;
    list($d,$i)=$dirs->read_contents($dirname);
    $this->directories=$d;
    $this->imagefiles=$i;
  }

  /** Utility function. Gets the first image from a directory.
   *  Recurses if necessary. Returns TRUE if $out_imagefiles is full. */
  function get_sample_imagefiles($max,&$out_samples)
  {
    // Find images.
    if(count($this->imagefiles)>0)
    {
      $this->imagefiles=array_values($this->imagefiles);
      for($i=0; $i<count($this->imagefiles); $i++)
      {
        $out_samples[] = $this->imagefiles[$i];
        if(count($out_samples) >= $max)
            return TRUE;
      }
    }
    // ...else recursively call into subdirs.
    if(count($this->directories)>0)
    {
      sort($this->directories);
      foreach($this->directories as $d)
      {
        $subalbum=new Album($this->dirname.'/'.$d);
        if( $subalbum->get_sample_imagefiles($max,$out_samples) )
            return TRUE;
      }
    }
    return FALSE;
  }

  /** Get the album thumbnail name. Make a new thumbnail if necessary. */
  function thumb()
  {
    global $dirs;
    $thumb_fname=$dirs->album($this->dirname).'/thumb.jpg';
    // If the thumbnail is up to date, just return it.
    if(!file_exists($thumb_fname))
    {
      // Generate the thumbnail.
      $samples=array();
      $this->get_sample_imagefiles(3,$samples);
      $sq=array();
      for($i=0; $i<3; $i++)
      {
        if($i < count($samples))
        {
          $ir=$samples[$i]->square();
          $sq[]=imagecreatefromjpeg( $ir->src );
        }
        else
        {
          $new=imagecreatetruecolor(75,75);
          $col=imagecolorallocate($new,240,240,240); // grey
          imagefill($new,0,0,$col);
          $sq[]=$new;
        }
      }
      $thumb=imagecreatetruecolor(113,75);
      $white=imagecolorallocate($thumb,255,255,255);
      imagefill($thumb,0,0,$white);
      //                 dst     src     dst-xy-src  dst-size-src
      imagecopyresampled($thumb, $sq[0], 0,0,   0,0, 75,75, 75,75 );
      imagecopyresampled($thumb, $sq[1], 76,0,  0,0, 37,37, 75,75 );
      imagecopyresampled($thumb, $sq[2], 76,38, 0,0, 37,37, 75,75 );
      // Save it.
      imagejpeg($thumb,$thumb_fname,85);
      imagedestroy($thumb);
      foreach($sq as $image)
          imagedestroy($image);
      chmod($thumb_fname,0666);
    }
    return $thumb_fname;
  }

} // end class Album

function table_dir_thumb($dir,$html,$is_here=false)
{
  $album=new Album($dir);
?>
  <table class="thumb"><tbody><tr>
  <td class="thumb" rowspan="2">
    <a href="<?php echo $dir
     ?>/?d=c"><img title="carousel" alt="carousel" width="113" height="75" src="<?php
         echo $album->thumb() ?>" /></a>
  </td><td class="detail">
    <?php echo $html; 
          if(count($album->imagefiles ))
            echo '<p>'.count($album->imagefiles ).' images</p>';
          if(!$is_here && count($album->directories))
            echo '<p>'.count($album->directories).' albums</p>'; ?>
  </td></tr><tr><td class="iconbox">
    <a class="on" href="<?php echo $dir
    ?>/?d=t"><img title="album" alt="album" src="<?php
      echo get_file(tsize_name().'.gif')?>" width="16" height="16" /></a>
  </td></tr></tbody></table>
<?php
  flush();
}

function doc_dir($directories)
{
  global $dirs;
  // Determine which page we should go to
  // [see the corresponding code in path_links()].
  if(array_key_exists('from_dir',$_GET))
    $referer_dir=array_search($_GET['from_dir'],$directories);
  else
    $referer_dir=0;
  $paged=new Paged(12,count($directories),$referer_dir);

?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>

<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
<title><?php echo hsc($dirs->path()) ?></title>
<link rel="stylesheet" href="<?php echo get_file('exa.css') ?>" type="text/css" />
<?php load_theme() ?>
</head>

<body>

<div id="wrap" class="dir">

<div id="path"><?php echo $dirs->path_links(FALSE) ?></div>

<?php if(count($dirs->imagefiles)): ?>

<div id="header">
<?php table_dir_thumb('.','<h1>'.$dirs->current_album_title().'</h1>',true) ?>
</div>
<hr />

<?php endif; ?>

<div id="main">
<?php

for($i=$paged->begin(); $i<$paged->end(); $i++)
  table_dir_thumb($directories[$i],'<h3>'.nice($directories[$i]).'</h3>');

?>
</div> <!-- main -->

<?php $paged->index('?') ?>
<?php status() ?>

</div> <!-- wrap -->

</body>
</html>
<?php } // doc_dir() -----------------------------------------------------------

?>
<?php // doc_latest.php

function all_comments(&$comments)
{
  $img_dirs=array('.');
  $exa_dirs=array();
  // Find all exa_dirs
  while(count($img_dirs))
  {
    $imgdir=array_shift($img_dirs);
    if($dhandle=opendir($imgdir))
    {
      while(FALSE !== ($fname=readdir($dhandle)))
      {
        if('.'==substr($fname,0,1))
            continue;
        $path=$imgdir.'/'.$fname;
        if('_exa'==$fname)
            $exa_dirs[]=$path;
        elseif(is_dir($path) && is_readable($path))
            $img_dirs[]=$path;
      } // end while
      closedir($dhandle);
    }
  }
  // Find all comment dirs.
  $cmt_dirs=array();
  foreach($exa_dirs as $exadir)
  {
    if($dhandle=opendir($exadir))
    {
      while(FALSE !== ($fname=readdir($dhandle)))
      {
        if('.'==substr($fname,0,1))
            continue;
        $cmtdir=$exadir.'/'.$fname.'/comments';
        if(is_dir($cmtdir) && is_readable($cmtdir))
            $cmt_dirs[]=$cmtdir;
      } // end while
      closedir($dhandle);
    }
  }
  // Find all comments.
  foreach($cmt_dirs as $cmtdir)
  {
    if($dhandle=opendir($cmtdir))
    {
      while(FALSE !== ($fname=readdir($dhandle)))
      {
        if('.'==substr($fname,0,1))
            continue;
        $cmt=$cmtdir.'/'.$fname;
        if(is_file($cmt) && is_readable($cmt))
            $comments[$fname]=$cmt;
      } // end while
      closedir($dhandle);
    }
  }
  // Sort by comment ID (i.e. by timestamp).
  ksort($comments);
}

function tr_comment(&$c)
{
  global $options,$is_admin;
  if($is_admin)
  {
    $lon='<label for="'.$c->id.'">';
    $loff='</label>';
  }
  else
  {
    $lon='';
    $loff='';
  }
  ?>

  <tr class="comment" id="tr_<?php echo $c->id ?>">
  <td class="square">
   <a href="<?php echo $c->link() ?>"><img width="75" height="75" src="<?php echo $c->square() ?>" /></a>
  </td>
  <td onclick="do_toggle($('<?php echo $c->id ?>'));return false">
   <p><?php echo $lon.$c->text.$loff ?></p>
   <p class="by"><?php echo $lon ?>posted by <span class="author"><?php echo $c->author ?></span>
    on <?php echo date($options->date_format,$c->time).$loff ?></p></td>
  <?php if($is_admin): ?><td>
   <input style="width:16px;height:16px" type="checkbox" id="<?php
    echo $c->id ?>" name="cpaths[]" value="<?php echo hsc($c->path)
    ?>" onclick="do_click(this)" /></td>
  <td><a class="control" href="#" title="mark to here for deletion" onclick="do_mark_to('<?php
    echo $c->id ?>');return false">&divide;</a>
  <?php endif ?>
  </tr>
  
  <?php
}

function doc_latest()
{
  global $dirs,$is_admin;
  $comments=array();
  all_comments($comments);
  $comments=array_reverse(array_values($comments));
  $paged=new Paged(50,count($comments));

?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>

<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
<title>Latest Comments &laquo; <?php echo hsc($dirs->path()) ?></title>
<link rel="stylesheet" href="<?php echo get_file('exa.css') ?>" type="text/css" />
<?php load_theme() ?>
<script type="text/javascript" src="<?php echo get_file('util.js') ?>"></script>
<script type="text/javascript" src="<?php echo get_file('latest.js') ?>"></script>
</head>

<body>

<div id="wrap" class="latest">

<div id="path"><?php echo $dirs->path_links(TRUE) ?></div>

<?php
  global $admin_message_text;
  echo $admin_message_text;
?>

<div id="header">
<h1>Latest Comments</h1>
<h2><?php echo $dirs->path() ?></h2>
</div>

<form method="POST" action="?d=l" id="main">
<input type="hidden" name="do" value="delcomments" />

<table><tbody>
<?php

if(count($comments))
{
  for($i=$paged->begin(); $i<$paged->end(); $i++)
  {
    $cmt=new Comment($comments[$i]);
    tr_comment($cmt);
  }
  if($is_admin)
      echo '<tr><th colspan="4"><input type="submit" value="delete" /></th></tr>'."\n";
}
else
{
  echo '<tr><td>No comments</td></tr>';
}

?>
</tbody></table>
</form> <!-- main -->

<?php $paged->index('?d=l&amp;') ?>
<?php status() ?>

</div> <!-- wrap -->

</body>
</html>
<?php } // doc_latest() --------------------------------------------------------

?>
<?php // main.php

/** Dies if $arr does not contain $keys. */
function assert_keys($keys,$arr)
{
  foreach($keys as $k)
    if(!array_key_exists($k,$arr))
      die('Missing key: '.$k);
}


/** Utility, deals with input (from $_POST).
 *  Strips slashes and makes it HTML-safe. */
function safe_input($s)
{
  return htmlentities(stripslashes( $s ), ENT_QUOTES);
}


function process_form()
{
  global $is_admin,$options,$dirs,$img;
  $do=$_POST['do'];
  if('setpasswd'==$do)
  {
    assert_keys(array('p1','p2'),$_POST);
    $p1=$_POST['p1'];
    $p2=$_POST['p2'];
    if(strlen($options->passwd) && !$is_admin)
      admin_message('You must log-in to set the password.');
    elseif(strlen($p1) < 8)
      admin_message('Your password must be at least eight characters long.');
    elseif($p1!=$p2)
      admin_message('You must type the same password twice.');
    else
    {
      $options->passwd=md5($p1.$options->secret);
      $options->save();
      setcookie('exa_passwd',$options->passwd,0,'/');  
      $is_admin=true;
    }
  }
  elseif(intval($options->comments) && 'addcomment'==$do)
  {
    // Dummy field 'url' must be left blank. Spam-bots will often complete all
    // fields. Also, insist that comments are made after a 4s wait, but
    // before 1hr has passed.
    // If a comment fails these tests, then it is silently ignored.
    // ?? REALLY SHOULD GIVE THE USER A SECOND CHANCE.
    global $options;
    assert_keys(array('cmt_author','cmt_txt','cmt_ts','cmt_chk','url'),$_POST);
    $author   =htmlentities(stripslashes( $_POST['cmt_author'] ));
    $comment  =htmlentities(stripslashes( $_POST['cmt_txt'] ));
    $dummy    =htmlentities(stripslashes( $_POST['url'] ));
    $timestamp=intval($_POST['cmt_ts']);
    $checksum =htmlentities(stripslashes( $_POST['cmt_chk'] ));
    $age=( time() - $timestamp );
    if( $age < $options->comment_wait_min )
        admin_message('Failed to save comment. You must wait at least '.
          $options->comment_wait_min.' seconds.');
    elseif( $age > $options->comment_wait_max )
        admin_message('Failed to save comment, because you waited too long. '
          . 'Refresh the page and try again.');
    elseif( strlen($author)<=0 )
        admin_message('You must enter your name.');
    elseif( strlen($comment)<=0 )
        admin_message('You must enter a comment.');
    elseif( strlen($dummy)!=0 )
        admin_message('You must leave the dummy field empty.');
    elseif( $checksum!=md5($options->secret.strval($timestamp)) )
        admin_message('Bad checksum.');
    else
        $dirs->imagefiles[$img]->addcomment($author,$comment);
  }
  elseif(!$is_admin)
  {
    die('Not logged in.');
  }
  
  // MUST BE LOGGED-IN TO GET PAST THIS POINT

  if('setprefs'==$do)
  {
    $options->comments=intval($_POST['comments']);
    $options->exif=intval($_POST['exif']);
    $options->reload=intval($_POST['reload']);
    $options->top_name=safe_input($_POST['top_name']);
    $options->home_name=safe_input($_POST['home_name']);
    $options->home_url=safe_input($_POST['home_url']);
    $options->slideshow_delay=max(0,1000*intval($_POST['slideshow_delay']));
    $options->theme=safe_input($_POST['theme']);
    $options->save();
    admin_message('Preferences saved.');
  }
  elseif('edittitle'==$do)
  {
    assert_keys(array('title_txt'),$_POST);
    $new_title=htmlentities(stripslashes( $_POST['title_txt'] ));
    if( !$dirs->imagefiles[$img]->set_title($new_title) )
        admin_message('Failed to rename '.$dirs->imagefiles[$img]->fname);
  }
  elseif('editcaption'==$do)
  {
    assert_keys(array('cap_txt'),$_POST);
    $caption=htmlentities(stripslashes( $_POST['cap_txt'] ));
    $dirs->imagefiles[$img]->setcaption($caption);
    admin_message('Caption saved.');
  }
  elseif('delcomment'==$do)
  {
    assert_keys(array('cid'),$_POST);
    $cid=safe_input($_POST['cid']);
    $dirs->imagefiles[$img]->delcomment($cid);
    admin_message('Comment deleted.');
  }
  elseif('delcomments'==$do)
  {
    assert_keys(array('cpaths'),$_POST);
    if(is_array($_POST['cpaths']))
        foreach($_POST['cpaths'] as $cpath)
            if(is_file($cpath) && 'comments'==basename(dirname($cpath)))
                unlink($cpath);
    admin_message('Comments deleted.');
  }
}

/** Force download of the current image
 *  (by setting MIME type to application/octet-stream). */
function download()
{
  assert_img();
  global $dirs,$img;
  $fname=$dirs->imagefiles[$img]->fname;
//  header('HTTP/1.1 200 OK');
  header('Date: ' . date("D M j G:i:s T Y"));
  header('Last-Modified: ' . date("D M j G:i:s T Y"));
  header("Content-Type: application/octet-stream");
  header("Content-Length: " . (string)(filesize($fname)) );
  header("Content-Transfer-Encoding: Binary");
  header('Content-Disposition: attachment; filename="'.basename($fname).'"' );
  readfile($fname);
  exit(0);
}


/** Empty document. Used for void-return XMLHttpRequest calls. */
function doc_empty()
{
  global $admin_message_text;
  ?>
  <html><body><?php echo $admin_message_text ?></body></html>
  <?php
}


/** Source code. */
function doc_code()
{
  if(array_key_exists('plain',$_GET))
  {
    header('Content-type: text/plain');
    echo file_get_contents('index.php');
  }
  else
  {
    highlight_file('index.php');
  }
}


function main()
{
  // Read globals
  global $dirs,$img,$isize,$tsize,$options;
  if(array_key_exists('f',$_GET) && count($dirs->imagefiles))
  {
    for($img=count($dirs->imagefiles)-1; $img>0; $img--)
        if($_GET['f']==$dirs->imagefiles[$img]->info['basename'])
            break;
  }
  else
  {
    $img=intval($_GET['img']);
  }
  $isize=get_set_cookie('isize',0,5,3);
  $tsize=get_set_cookie('tsize');

  if(array_key_exists('do',$_POST))
    process_form();
  elseif(array_key_exists('dl',$_GET))
    download();

  $doc=$_GET['d'];
  if(0==strlen($options->passwd) || 'a'==$doc)
    doc_admin();
  elseif('e'==$doc)
    doc_empty();
  elseif('l'==$doc)
    doc_latest();
  elseif('p'==$doc)
    doc_code();
  elseif(0==count($dirs->imagefiles)) // No images
    doc_dir($dirs->directories);
  elseif('x'==$doc)
    doc_xml($dirs->imagefiles);
  elseif('s'==$doc)
    doc_single($dirs->imagefiles);
  elseif('c'==$doc)
    doc_carousel($dirs->imagefiles);
  elseif('t'==$doc || 0==count($dirs->directories))
    doc_thumb($dirs->imagefiles);
  else
    doc_dir($dirs->directories);
}
main();

?>
<?php // EMBEDDED.php

/** Calculates the address of the file (in top level _exa directory.
 * Writes the file if it does not already exist. */
function get_file($name)
{
  global $dirs,$options;
  $result=$dirs->top_album().'/'.$name;
  if($options->reload || !file_exists($result))
  {
    $handle=fopen($result,'wb');
    if('rew.png'==$name)
      fwrite($handle,base64_decode(<<<EOFEOF
iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABmJLR0QAAAAA
AAD5Q7t/AAAACXBIWXMAAA6cAAAOnAEHlFPdAAAAB3RJTUUH1wIKFSY0LhUj
OQAAAUlJREFUOMvFk79LAmEYxz+nSVJx6GCJU3KIIk1FSzcYJERhQdnU0OCg
LU03FIJCTUJj/0C6SENDBk39gIqUpsZEDteizUkQeVvyuNOOHIS+y/Py+b58
eV7e54FRKB3L2Fkp4G0o3hciAbl0LPMAdG24AHD0HFVRCfsjAOPAVdgfOVEV
dRlw2HAAxnoH76SX+sf7LHC/pKhBbVXD5XQB2HJLwObZxorslm+35rdJLiQt
z5PdcvM33gsIALueCc9pLpEnNBMyTCEEa3PrxKNxC+8PyCs+JZNNZPFNTVtM
SZLYWUwOcLMcwL7+pWvahUbjs2ExhRA81p8GuFnOn1ptd9rPNb26J4BoIGp0
cHR5SE2vYuYA5dcywLHxjZWD67tWuxUsvpw3CzcFOt2OcdmOW1RMlcxzUAn7
I6KYKglA2PFhJ1H8yUeyC/+mb3QYf88q2/KcAAAAAElFTkSuQmCC
====
EOFEOF
      ));
    //###############################################################
    elseif('thumb.gif'==$name)
      fwrite($handle,base64_decode(<<<EOFEOF
R0lGODlhEAAQAOMKAAAAAJ11v791db91p791sHWuv7+bdXW/pKC/db+5df//
/////////////////////yH+MENyZWF0ZWQgYnkgYWxleC5naW1wQGZpcmV0
cmVlLm5ldCB3aXRoIFRoZSBHSU1QLgAh+QQBCgAPACwAAAAAEAAQAAAEQxBI
8N6k9hKCQQhdUUhb94XjVamZVL1wLM8xa1/H0SFIlySS3K63+l06k1WSxmzC
bq6WxGDoCASdwWBaXV2zWyhSEgEAOw==
====
EOFEOF
      ));
    //###############################################################
    elseif('browse.gif'==$name)
      fwrite($handle,base64_decode(<<<EOFEOF
R0lGODlhEAAQAMIFAAAAAJ11v791p3Wuv7+5df///////////yH+MENyZWF0
ZWQgYnkgYWxleC5naW1wQGZpcmV0cmVlLm5ldCB3aXRoIFRoZSBHSU1QLgAh
+QQBCgAHACwAAAAAEAAQAAADPni6rPDgwPncGBGEzUOUV9Zx3ySN29doqMqw
YwmdrUMQYi3deezMsI5rESQ1HCjPr1JMOQSCDGUmgQ6PEkgCADs=
====
EOFEOF
      ));
    //###############################################################
    elseif('latest.js'==$name)
      fwrite($handle,<<<EOFEOF
function do_click(e)
{
  var tr=e.parentNode.parentNode;
  if(tr.cells.length>2)
  {
    if(e.checked)
      tr.cells[1].style.backgroundColor='red';
    else
      tr.cells[1].style.backgroundColor='transparent';
  }
}

function do_mark_to(id)
{
  var to=true;
  var inputs=document.getElementsByTagName('input');
  for(var i=0; i<inputs.length; i++)
  {
    if(inputs[i].type=='checkbox')
    {
      if(inputs[i].checked!=to)
      {
        inputs[i].checked=to;
        do_click(inputs[i]);
      }
      if(inputs[i].id==id)
        to=false;
    }
  }
}

function do_toggle(e)
{
  e.checked=!e.checked;
  do_click(e);
}

window.onload=function()
{
  // Handle page refreshes where the checkbox has been checked, but the
  // form has not been submitted.
  var inputs=document.getElementsByTagName('input');
  for(var i=0; i<inputs.length; i++)
  {
    if(inputs[i].type=='checkbox' && inputs[i].checked)
      do_click(inputs[i]);
  }
}

EOFEOF
      );
    //###############################################################
    elseif('load.gif'==$name)
      fwrite($handle,base64_decode(<<<EOFEOF
R0lGODlhEAAQAMIHAL82UdQkSLpWasSTnb91hMmxtsO9vv///yH/C05FVFND
QVBFMi4wAwEAAAAh+QQFAQAHACwAAAAAEAAQAAADQ3i6B8CQORdXCG0eIeC9
2cZ11seMZBlxjGFUC0EcrgvLcv1W+GzDB1lrxxgMILqi8bhIFgqHJbP5ej6j
04gVClxcIwkAIfkEBQEABwAsAAAAABAAEAAAAz94uifCkDkXFwBtHkLgvdnG
ddbHjGQZcUwQVMswHK4Ly3L9VvhswwcZcFEoDIlFI8xgOCSVESbTCY1Kj4pp
JAEAIfkEBQEABwAsAAAAABAADgAAAzt4ukfEkDkXlxBtnjHgvdnGddbHjGQZ
cQwAVEtRHK4Ly3L9VvhswwcZIxCIGAwQIpFxPA6VzGayCHEqEgAh+QQFAQAH
ACwAAAAAEAAQAAADPni6N8OQORcXIW2eUuC92cZ11seMZBlxjCBUi2EcrgvL
cv1W+GzDBxkDAAAOiUXjAVkMBIzEg9OplE6r1koCACH5BAUBAAcALAAAAAAO
ABAAAAM8eLpXxVA5F88YbR5j1r3ZxnXWx4xkGXEKQVSM68KtTNc3IwhRECy7
HcPnUwR5AMCB+DMik8piBKq8JSEJACH5BAUBAAcALAAAAAAQABAAAAM+eLpn
xpA5F1cpbdZzb95cBzLeeAzDGAQnmlbr6r5RzKIquxBEBAAQHo/x+zGEPYHg
UAQek8qlcRNdmg7KSgIAIfkEBQEABwAsAAACABAADgAAAz54aqZ+IbzD2Ivx
1eaw1Nz1KUUxTQBwlOWppClrurDauq/qDMMpCBMe7/H7PYQ9AuFQBB6TyqUR
F13iHkpXAgAh+QQBAQAHACwAAAAAEAAQAAADPni6F8GQORfjfADURXPejKeB
y7cYBikIB4pu6+qmVcy+4MoURUQQEB6P8fvthIfB4FAEHpPKpXETXZIUykoC
ADs=
====
EOFEOF
      ));
    //###############################################################
    elseif('album.gif'==$name)
      fwrite($handle,base64_decode(<<<EOFEOF
R0lGODlhEAAQAMIFAAAAAJ11v3Wuv6C/db+5df///////////yH+MENyZWF0
ZWQgYnkgYWxleC5naW1wQGZpcmV0cmVlLm5ldCB3aXRoIFRoZSBHSU1QLgAh
+QQBCgAHACwAAAAAEAAQAAADPgi6cMcsSNmeuFdN6gDOwBZU3qdt5CecU2pC
Sgc7dG3fuA2T++D7JIJQqPgBO0MiwDgIJotG5xD6kypnD1gCADs=
====
EOFEOF
      ));
    //###############################################################
    elseif('single.gif'==$name)
      fwrite($handle,base64_decode(<<<EOFEOF
R0lGODlhEAAQAKECAAAAAKC/df///////yH+MENyZWF0ZWQgYnkgYWxleC5n
aW1wQGZpcmV0cmVlLm5ldCB3aXRoIFRoZSBHSU1QLgAh+QQBCgACACwAAAAA
EAAQAAACIZSPqYvgD6MLtFo7r6Z53+5VYBiMoemhm6qxnwQ/zEwnBQA7
====
EOFEOF
      ));
    //###############################################################
    elseif('bigger.gif'==$name)
      fwrite($handle,base64_decode(<<<EOFEOF
R0lGODlhEAAQAIQTAAAAAAUFBX9/f511v791db91p791sHWuv7+bdXW/pKC/
db+5dc6539+5ud+517nW37nf0c/fud/cuf//////////////////////////
/////////////////////////yH+MENyZWF0ZWQgYnkgYWxleC5naW1wQGZp
cmV0cmVlLm5ldCB3aXRoIFRoZSBHSU1QLgAh+QQBCgAfACwAAAAAEAAQAAAF
ceAnfkBpnuVIGsayEMQgD4AKsC4803aS6ANgbQTwAYU2BEKmUBQKMIBgKggo
mU4oQepwNACuJksZFXS/YcUYUYZAGADY4dD0uaRuuJyusC+kEoEATUo+c02A
goQIhnxSEREPKJOPkQ+QAioqVJyZmiIhADs=
====
EOFEOF
      ));
    //###############################################################
    elseif('smaller.gif'==$name)
      fwrite($handle,base64_decode(<<<EOFEOF
R0lGODlhEAAQAIQVAAAAAH9/f5x1vp11v751db91db91sHWtvnWuv3W/pKC/
db+5dc6539+5ud+509+517nW39/Nubnf0c/fud/cuf//////////////////
/////////////////////////yH+MENyZWF0ZWQgYnkgYWxleC5naW1wQGZp
cmV0cmVlLm5ldCB3aXRoIFRoZSBHSU1QLgAh+QQBCgAfACwAAAAAEAAQAAAF
euAnfkFpnuVIThMESVIUsYEasC4s0zZFwThIo1EbBXzAlpBoG8oeD5YvAKha
qxGolEJdLBAIhQLgYDEYsu73ICab0RGqwSD2AhrnIUxOV9jxDHoSVAUFAwIJ
CQCBDT5QhIaIioyOD1QDAwUEcwAoJgCYhZwqKldXpCMhADs=
====
EOFEOF
      ));
    //###############################################################
    elseif('play.png'==$name)
      fwrite($handle,base64_decode(<<<EOFEOF
iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABmJLR0QA/wD/
AP+gvaeTAAAACXBIWXMAAA6cAAAOnAEHlFPdAAAAB3RJTUUH1wIBESgRK5r1
LQAAAZFJREFUOMulU0sohFEU/u6df6akPCcJ/2Q8FhaTQqHkMUpsFClRiBKm
rLiajd1sPBYsiKZRSsZzsJwFl62FLJStPLIha/TPsZkZd5iZJr46m3PO/e75
zgP4ASICESGOn4X9GgAWNs6VnGsALvWRT0j4hPzJZYiAJBGQAEAcAKQ4BoDK
PHvZitVmf2eM6fgDaPrglBr6R0izWKi0pv4wgQSLIoFxVTPjHPU9Axhc9OLt
+am7sMLx0TYx5VR7QkSfpIDHKyW3yIbhpQ2UVNeZLzbXTh3O9r2Zo/Moh5rL
1CrDjYnBy8MdDj1umDTzXXp2TmefZ/lmobslSsIBQMithI2x6sUYW99BRWOr
jXPTCQAmgwIyKBCZaVJEKggZxm1WfkFfyhJChoHLk11cBrZRXtuw3zHp7p3v
aibGWMwHWnQCSuD18R7Hc7NIy8j8bBoabw+uLp6pm5podX/tQSSWzGK48uxl
lKsXvwPQU72RCEwArgC41EQpBKQQvwik8EMK//cUpFcYAKpaRhfwbyQ557j+
Lz+j6kr4Uv4qAAAAAElFTkSuQmCC
====
EOFEOF
      ));
    //###############################################################
    elseif('stop.gif'==$name)
      fwrite($handle,base64_decode(<<<EOFEOF
R0lGODlhEAAQAKECAAAAAL91df///////yH+MENyZWF0ZWQgYnkgYWxleC5n
aW1wQGZpcmV0cmVlLm5ldCB3aXRoIFRoZSBHSU1QLgAh+QQBCgACACwAAAAA
EAAQAAACKJSPqavgDx0CodoL5t0hH8pZngGGnWaeX6qS7CiUISxz9DtFOsP3
SwEAOw==
====
EOFEOF
      ));
    //###############################################################
    elseif('fwd.png'==$name)
      fwrite($handle,base64_decode(<<<EOFEOF
iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAABmJLR0QAAAAA
AAD5Q7t/AAAACXBIWXMAAA6cAAAOnAEHlFPdAAAAB3RJTUUH1wIKFSQQICCl
agAAASZJREFUOMvN0s8rw3Ecx/HnZ99psm+LOGgp3/lRIlGUWkqxlNUuODlI
3wsXp50clFJO8kfsuOLgpnBRkjJTpFbz3VoOclCraW30cWG1fb7fITt4XT6f
Hp8+r8O7N/wx2ueZBErAdc27kyuRq4tTF8AmIH7gakFs25QDRqcEDgHPN16J
6+vS5vOysz5PcKQ3AtwDRj1XCgCa3BobZpiVSDDg8zZbdXymdohbS3MTlaLB
Hj/DfV1oQjA+ZCCEqPLLW2u5WCoXgIxtAUB7q063vwO9xaP45Gg/dw+Psy/5
V93lNNVU9omzRAoppeLRvTjp3HMUWHPbfd4/vuLgNEG+UGQhNGbnIeAEoKqg
/PbObuyI85u0BUwDloNnGrYHDdvEJGD+wv9RPgDw7X7SoJ+aTwAAAABJRU5E
rkJggg==
====
EOFEOF
      ));
    //###############################################################
    elseif('exa.css'==$name)
      fwrite($handle,<<<EOFEOF
body {
 font-family:"Trebuchet MS",sans-serif;
 font-size:4mm;
}
div, form, table {
 margin:0;
 padding:0;
}
img {
 margin:0;
 border:none;
 padding:0;
}

h1,h2 {
 margin:0;
}

button {
 margin:2px 0 2px 0;
 border:solid 2px gray;
 background-color:#fafafa;
 text-align:center;
 font-size:12pt;
 font-weight:bold;
}
button.off {
 border:solid 2px #ccc;
 background-color:#f4f4f4;
 color:#ccc;
}

/* Set the vertical spacing. #main has no margin, everything else
 * sits above or below it. */
#path, #header, #browser {
 margin-bottom:8px;
}
#pages, #status {
 margin-top:8px;
}

/* DIV wrapper for content. */
#wrap {
 width:860px;
 margin-left:auto;
 margin-right:auto;
 padding-left:20px;
 padding-right:20px;
}

#path {
 padding:2px 5px;
 background-color:#f4f4f4;
}

#download {
 float:right;
}

/* banner header */
#header {
 border:none;
 width:100%;
}
#header tr {
 vertical-align:bottom;
}
#header .bottom_right {
 text-align:right;
}

/* #thumbs
 * album, thumbnail view.
 */
div#thumbs {
 width:560px;
 text-align:left;
 margin-left:auto;
 margin-right:auto;
}
div#thumbs img {
 display:inline;
}

table#thumbs {
 margin-left:auto;
 margin-right:auto;
}
table#thumbs td {
 padding-left:10px;
 padding-right:10px;
 width:240px;
 vertical-align:top;
}
table#thumbs h3 {
 margin-bottom:3px;
}
table#thumbs p {
 margin-top:0.5ex;
}
#thumbs a {
 text-decoration:none;
}
#thumbs p.caption {
  width:auto;
  margin:0;
  padding:0;
}
#thumbs p.num_comments {
  float:none;
  margin:0;
  padding:0;
  font-size:smaller;
}


/* Carousel */
.carousel #main {
 width:516px;
 float:left;
}
#browser {
 margin:0;
 border:solid #ccc 1px;
 padding:4px 6px 4px 6px;
 width:330px;
 float:left;
 background:#f4f4f4;
}
#browser .thumb {
 margin:2px 0 2px 0;
 border:none;
 width:330px;
 height:75px;
}
#browser .thumb a {
 float:left;
 clear:left;
 margin-right:5px;
 width:75px;
 height:75px;
}
#browser .thumb div {
 float:left;
 margin:0;
 padding:0;
 width:250px;
 height:75px;
 overflow:hidden;
}
#browser .thumb h3,
#browser .thumb p {
 margin:0px;
 padding:0px;
}
#browser .thumb p {
 font-size:0.8em;
}
#browser div.here {
 background:#fafafa;
}
#browser button {
 width:75px;
}
#browser .img_last {
 border-bottom:dashed 1px black;
}
.date_time {
 color:gray;
}
.num_comments {
 float:right;
 margin-left:1ex;
 margin-right:1ex;
 color:gray;
}


/* .iconbox
 * Container for icons. Formats the icons and implements link hover.
 */
.iconbox * {
 vertical-align:middle;
}
.iconbox img, .iconbox a.on img {
 margin:1px;
 padding:3px;
 filter:alpha(opacity=100);
 -moz-opacity:1;
 opacity:1;
}
.iconbox a img, .iconbox a.off:hover img {
 filter:alpha(opacity=30);
 -moz-opacity:.3;
 opacity:.3;
}
.iconbox a:hover img {
 background-color:white;
 margin:0px;
 border:solid 1px #ccc;
 filter:alpha(opacity=100);
 -moz-opacity:1;
 opacity:1;
}

.dir #pages, .dir #status, .carousel #status {
 float:left;
 width:100%;
}

/* directory thumbnails */
.dir table.thumb {
 margin:1px;
 border:none;
 border-collapse:collapse;
 height:75px;
}
.dir table.thumb td {
 vertical-align:top;
}
.dir table.thumb td.thumb {
 width:118px;
 padding-bottom:5px;
}
.dir table.thumb td.iconbox {
 vertical-align:bottom;
}
.dir table.thumb td.detail * {
 margin:0px;
 padding:0px;
}
.dir #header table.thumb {
 margin-top:10px ;
}
.dir #main table.thumb {
 margin-right:15px;
 float:left;
 width:413px;
}


/* #single
 * Contains the main image on 'single' pages.
 */
#single {
 text-align:center
}
#single img {
 margin-left:auto;
 margin-right:auto;
}

/* #pages
 * Index-bar at the bottom of paged 'directory' and 'album' pages.
 */
#pages {
 padding:2px 5px;
 background-color:#f4f4f4;
 text-align:center;
 font-size:12pt;
 font-weight:bold;
}
#pages span, #pages a {
 margin-left:2px;
 margin-right:2px;
 padding-left:5px;
 padding-right:5px;
}
#pages span.off {
 border:solid 1px #ccc;
 color:#ccc;
 background-color:#f4f4f4;
}
#pages span.here {
 border:solid 1px black;
 color:white;
 background-color:#a33;
}
#pages a {
 border:solid 1px #ccc;
 color:black;
 background-color:white;
 text-decoration:none;
}
#pages a:hover {
 background-color:#ccc;
}

/* #status */
#status {
 margin-left:auto;
 margin-right:auto;
 text-align:center;
 font-size:x-small;
}
.alert {
 border:solid red 2px;
 padding:3px;
}

/* title */
h1, .title #title_txt {
 font-family:"Trebuchet MS",sans-serif;
 font-size:2em;
 font-weight:bold;
}

/* caption */
.caption { margin:1em 0 }
.caption #cap_txt {
 margin:0px;
 padding:0px
}
p.caption, .caption #cap_txt {
 width:400px;
 font-family:"Trebuchet MS",sans-serif;
 font-size:1em;
}
#cap_ctrl {
 position:relative; /* Fixes IE idiocy. */
 padding:0 3px;
 background-color:#f4f4f4;
 border:solid 1px gray;
 font-size:x-small;
 text-decoration:none;
}

/* exif */
.exif {
 font-size:smaller;
 color:gray;
}

/* comments */
.comments { margin:1em 0 }
.comment  { margin-bottom:0.5em }
.comment #cmt_txt {
 margin:1ex 0;
 padding:0px
}
div.comment, .comment #cmt_txt {
 width:400px;
 font-family:"Trebuchet MS",sans-serif;
 font-size:1em;
}
a.control {
 position:relative; /* Fixes IE idiocy. */
 padding:0 3px;
 background-color:#f4f4f4;
 border:solid 1px gray;
 font-size:x-small;
 font-weight:normal;
 text-decoration:none;
}
.comment p { margin:0 }
.comment p.by { font-size:smaller; color:gray; }
.author { font-weight:bold }
.comment td { vertical-align:top }
.latest #main {
 margin-left:auto;
 margin-right:auto;
 width:560px;
}
.latest th { text-align:right }
.latest td.square { width:75px }
.latest .control {
 display:block;
 font-size:16px;
 width:1em;height:1em;
 text-align:center;
 vertical-align:baseline;
 padding:0px;
 margin:3px;
}

/* fade (zoom:1 fixes IE hasLayout property.) */
.fade9 { zoom:1;filter:alpha(opacity=90); -moz-opacity:.9; opacity:.9; }
.fade8 { zoom:1;filter:alpha(opacity=80); -moz-opacity:.8; opacity:.8; }
.fade7 { zoom:1;filter:alpha(opacity=70); -moz-opacity:.7; opacity:.7; }
.fade6 { zoom:1;filter:alpha(opacity=60); -moz-opacity:.6; opacity:.6; }
.fade5 { zoom:1;filter:alpha(opacity=50); -moz-opacity:.5; opacity:.5; }
.fade4 { zoom:1;filter:alpha(opacity=40); -moz-opacity:.4; opacity:.4; }
.fade3 { zoom:1;filter:alpha(opacity=30); -moz-opacity:.3; opacity:.3; }
.fade2 { zoom:1;filter:alpha(opacity=20); -moz-opacity:.2; opacity:.2; }
.fade1 { zoom:1;filter:alpha(opacity=10); -moz-opacity:.1; opacity:.1; }
.fade0 { zoom:1;filter:alpha(opacity=0); -moz-opacity:0; opacity:0; }

EOFEOF
      );
    //###############################################################
    elseif('xmlhttprequest.js'==$name)
      fwrite($handle,<<<EOFEOF
/*

Cross-Browser XMLHttpRequest v1.2
=================================

Emulate Gecko 'XMLHttpRequest()' functionality in IE and Opera. Opera requires
the Sun Java Runtime Environment <http://www.java.com/>.

by Andrew Gregory
http://www.scss.com.au/family/andrew/webdesign/xmlhttprequest/

This work is licensed under the Creative Commons Attribution License. To view a
copy of this license, visit http://creativecommons.org/licenses/by-sa/2.5/ or
send a letter to Creative Commons, 559 Nathan Abbott Way, Stanford, California
94305, USA.

Attribution: Leave my name and web address in this script intact.

Not Supported in Opera
----------------------
* user/password authentication
* responseXML data member

Not Fully Supported in Opera
----------------------------
* async requests
* abort()
* getAllResponseHeaders(), getAllResponseHeader(header)

*/
// IE support
if (window.ActiveXObject && !window.XMLHttpRequest) {
  window.XMLHttpRequest = function() {
    var msxmls = new Array(
      'Msxml2.XMLHTTP.5.0',
      'Msxml2.XMLHTTP.4.0',
      'Msxml2.XMLHTTP.3.0',
      'Msxml2.XMLHTTP',
      'Microsoft.XMLHTTP');
    for (var i = 0; i < msxmls.length; i++) {
      try {
        return new ActiveXObject(msxmls[i]);
      } catch (e) {
      }
    }
    return null;
  };
}
// Gecko support
/* ;-) */
// Opera support
if (window.opera && !window.XMLHttpRequest) {
  window.XMLHttpRequest = function() {
    this.readyState = 0; // 0=uninitialized,1=loading,2=loaded,3=interactive,4=complete
    this.status = 0; // HTTP status codes
    this.statusText = '';
    this._headers = [];
    this._aborted = false;
    this._async = true;
    this._defaultCharset = 'ISO-8859-1';
    this._getCharset = function() {
      var charset = _defaultCharset;
      var contentType = this.getResponseHeader('Content-type').toUpperCase();
      val = contentType.indexOf('CHARSET=');
      if (val != -1) {
        charset = contentType.substring(val);
      }
      val = charset.indexOf(';');
      if (val != -1) {
        charset = charset.substring(0, val);
      }
      val = charset.indexOf(',');
      if (val != -1) {
        charset = charset.substring(0, val);
      }
      return charset;
    };
    this.abort = function() {
      this._aborted = true;
    };
    this.getAllResponseHeaders = function() {
      return this.getAllResponseHeader('*');
    };
    this.getAllResponseHeader = function(header) {
      var ret = '';
      for (var i = 0; i < this._headers.length; i++) {
        if (header == '*' || this._headers[i].h == header) {
          ret += this._headers[i].h + ': ' + this._headers[i].v + '\\n';
        }
      }
      return ret;
    };
    this.getResponseHeader = function(header) {
      var ret = getAllResponseHeader(header);
      var i = ret.indexOf('\\n');
      if (i != -1) {
        ret = ret.substring(0, i);
      }
      return ret;
    };
    this.setRequestHeader = function(header, value) {
      this._headers[this._headers.length] = {h:header, v:value};
    };
    this.open = function(method, url, async, user, password) {
      this.method = method;
      this.url = url;
      this._async = true;
      this._aborted = false;
      this._headers = [];
      if (arguments.length >= 3) {
        this._async = async;
      }
      if (arguments.length > 3) {
        opera.postError('XMLHttpRequest.open() - user/password not supported');
      }
      this.readyState = 1;
      if (this.onreadystatechange) {
        this.onreadystatechange();
      }
    };
    this.send = function(data) {
      if (!navigator.javaEnabled()) {
        alert("XMLHttpRequest.send() - Java must be installed and enabled.");
        return;
      }
      if (this._async) {
        setTimeout(this._sendasync, 0, this, data);
        // this is not really asynchronous and won't execute until the current
        // execution context ends
      } else {
        this._sendsync(data);
      }
    }
    this._sendasync = function(req, data) {
      if (!req._aborted) {
        req._sendsync(data);
      }
    };
    this._sendsync = function(data) {
      this.readyState = 2;
      if (this.onreadystatechange) {
        this.onreadystatechange();
      }
      // open connection
      var url = new java.net.URL(new java.net.URL(window.location.href), this.url);
      var conn = url.openConnection();
      for (var i = 0; i < this._headers.length; i++) {
        conn.setRequestProperty(this._headers[i].h, this._headers[i].v);
      }
      this._headers = [];
      if (this.method == 'POST') {
        // POST data
        conn.setDoOutput(true);
        var wr = new java.io.OutputStreamWriter(conn.getOutputStream(), this._getCharset());
        wr.write(data);
        wr.flush();
        wr.close();
      }
      // read response headers
      // NOTE: the getHeaderField() methods always return nulls for me :(
      var gotContentEncoding = false;
      var gotContentLength = false;
      var gotContentType = false;
      var gotDate = false;
      var gotExpiration = false;
      var gotLastModified = false;
      for (var i = 0; ; i++) {
        var hdrName = conn.getHeaderFieldKey(i);
        var hdrValue = conn.getHeaderField(i);
        if (hdrName == null && hdrValue == null) {
          break;
        }
        if (hdrName != null) {
          this._headers[this._headers.length] = {h:hdrName, v:hdrValue};
          switch (hdrName.toLowerCase()) {
            case 'content-encoding': gotContentEncoding = true; break;
            case 'content-length'  : gotContentLength   = true; break;
            case 'content-type'    : gotContentType     = true; break;
            case 'date'            : gotDate            = true; break;
            case 'expires'         : gotExpiration      = true; break;
            case 'last-modified'   : gotLastModified    = true; break;
          }
        }
      }
      // try to fill in any missing header information
      var val;
      val = conn.getContentEncoding();
      if (val != null && !gotContentEncoding) this._headers[this._headers.length] = {h:'Content-encoding', v:val};
      val = conn.getContentLength();
      if (val != -1 && !gotContentLength) this._headers[this._headers.length] = {h:'Content-length', v:val};
      val = conn.getContentType();
      if (val != null && !gotContentType) this._headers[this._headers.length] = {h:'Content-type', v:val};
      val = conn.getDate();
      if (val != 0 && !gotDate) this._headers[this._headers.length] = {h:'Date', v:(new Date(val)).toUTCString()};
      val = conn.getExpiration();
      if (val != 0 && !gotExpiration) this._headers[this._headers.length] = {h:'Expires', v:(new Date(val)).toUTCString()};
      val = conn.getLastModified();
      if (val != 0 && !gotLastModified) this._headers[this._headers.length] = {h:'Last-modified', v:(new Date(val)).toUTCString()};
      // read response data
      var reqdata = '';
      var stream = conn.getInputStream();
      if (stream) {
        var reader = new java.io.BufferedReader(new java.io.InputStreamReader(stream, this._getCharset()));
        var line;
        while ((line = reader.readLine()) != null) {
          if (this.readyState == 2) {
            this.readyState = 3;
            if (this.onreadystatechange) {
              this.onreadystatechange();
            }
          }
          reqdata += line + '\\n';
        }
        reader.close();
        this.status = 200;
        this.statusText = 'OK';
        this.responseText = reqdata;
        this.readyState = 4;
        if (this.onreadystatechange) {
          this.onreadystatechange();
        }
        if (this.onload) {
          this.onload();
        }
      } else {
        // error
        this.status = 404;
        this.statusText = 'Not Found';
        this.responseText = '';
        this.readyState = 4;
        if (this.onreadystatechange) {
          this.onreadystatechange();
        }
        if (this.onerror) {
          this.onerror();
        }
      }
    };
  };
}
// ActiveXObject emulation
if (!window.ActiveXObject && window.XMLHttpRequest) {
  window.ActiveXObject = function(type) {
    switch (type.toLowerCase()) {
      case 'microsoft.xmlhttp':
      case 'msxml2.xmlhttp':
      case 'msxml2.xmlhttp.3.0':
      case 'msxml2.xmlhttp.4.0':
      case 'msxml2.xmlhttp.5.0':
        return new XMLHttpRequest();
    }
    return null;
  };
}
EOFEOF
      );
    //###############################################################
    elseif('util.js'==$name)
      fwrite($handle,<<<EOFEOF
//=====================================================================
// Event Listener
// by Scott Andrew - http://scottandrew.com
// edited by Mark Wubben, <useCapture> is now set to false
//=====================================================================
function addEvent(obj, evType, fn)
{
  if(obj.addEventListener){
    obj.addEventListener(evType, fn, false); 
    return true;
  } else if (obj.attachEvent){
    var r = obj.attachEvent('on'+evType, fn);
    return r;
  } else {
    return false;
  }
}

/** Constants to aid readability of onclick handlers. */
var ONCLICK_OK     =false; // Do NOT execute href.
var ONCLICK_FAILED =true;  // DO execute href.

/** Shorthand for document.getElementById. Borrowed from Prototype.js */
function $(id){ return document.getElementById(id); }

/** Returns parameter p if it's set, else the delfault value d. */
function dflt(p,d)
{
  return p==null? d: p;
}

/** Move 'e' to precede 'before'. */
function move_before(e,before)
{
  if(e && before)
  {
    e.parentNode.removeChild(e);
    before.parentNode.insertBefore(e,before);
  }
}

/** Helper class for generating POST parameters.
 *  If the optional parameter 'form' is set, then the object is initialised from
 *  the form's fields.
 */
function Params(form)
{
  this.value='';
  if(form!=null)
    for(var i=0; i<form.elements.length; i++)
      this.add(form.elements[i].name,form.elements[i].value);
}
Params.prototype.add=function(k,v)
{
  if(this.value.length>0)
    this.value+='&';
  this.value+=k+'='+encodeURI(v);
}

function getFirstChildByTagName(e,tagName)
{
  if(e)
  {
    var arr=e.getElementsByTagName(tagName);
    if(arr.length)
      return arr[0];
  }
  return false;
}

/** Gets all of the TEXT and CDATA from an element, concatenated into a
 *  single string. */
function getInnerText(e,result)
{
  result=dflt(result,'');
  for(var i=0; i<e.childNodes.length; i++)
  {
    var nt=e.childNodes[i].nodeType;
    if(nt==3 /* TEXT_NODE */ || nt==4 /* CDATA_SECTION_NODE */)
    {
      if(result.length==0)
        result+=' ';
      result+=e.childNodes[i].nodeValue;
    }
    else if(nt==1 /* ELEMENT_NODE */)
    {
      result=getInnerText(e.childNodes[i],result);
    }
  }
  return result;
}

/** Submit an Ajax request. Options are:
 *   oncomplete: callback function. Passed the xml document.
 *   method: 'GET', 'POST' (default)
 *   data: data for POST request
 *   form: form from which to calculate data (ignored if data is set).
 */
function ajax(url,options)
{
  if(options==null)
    options=new Object();
  var req=new XMLHttpRequest();
  if(!req)
    return false;
  // method
  if(options.method)
    req.open(options.method,url,true);
  else
    req.open('POST',url,true);
  // data, form
  var data=null;
  if(options.data)
  {
    data=options.data;
  }
  else if(options.form)
  {
    var params=new Params(options.form);
    data=params.value;
  }
  if(data)
  {
    req.setRequestHeader("Content-type","application/x-www-form-urlencoded");
    req.setRequestHeader("Content-length",data.length);
  }
  // oncomplete
  if(options.oncomplete)
  {
    var oncomplete=options.oncomplete;
    req.onreadystatechange=
      function()
      {
        if(!(req.readyState==4 && req.status==200))
          return;
        oncomplete(req.responseXML.documentElement);
      }
  }
  req.setRequestHeader('If-Modified-Since','Sat, 1 Jan 2000 00:00:00 GMT');
  req.send(data);
  return true;
}

/** Helper function that submits a form through Ajax. */
function ajax_form(form,options)
{
  if(options==null)
    options=new Object();
  options.form=form;
  return ajax(form.action,options);
}

/** Fade out then remove an element. */
function fade_out(e,hide)
{
  var className=dflt(e.className,'');
  var trans=10;
  function evt_fade_out()
  {
    trans-=2;
    if(trans<=0)
    {
      if(hide)
      {
        e.className=className;
        e.style.display='none';
      }
      else
      {
        e.parentNode.removeChild(e);
      }
    }
    else
    {
      e.className='fade'+trans+' '+className;
      setTimeout(evt_fade_out,100);
    }
  }
  evt_fade_out();
}

/** Fade in an element. Optionally set its style.display */
function fade_in(e,display)
{
  if(display)
    e.style.display=display;
  var className=dflt(e.className,'');
  var trans=0;
  function evt_fade_in()
  {
    if(trans>=10)
    {
      e.className=className;
    }
    else
    {
      e.className='fade'+trans+' '+className;
      trans+=2;
      setTimeout(evt_fade_in,100);
    }
  }
  evt_fade_in();
}

/** Finds Absolute Y position of an element. */
function findPosY(obj)
{
  var curtop = 0;
  if(obj.offsetParent)
      while(1)
      {
        curtop += obj.offsetTop;
        if(!obj.offsetParent)
          break;
        obj = obj.offsetParent;
      }
  else if(obj.y)
      curtop += obj.y;
  return curtop;
}

/** Cross-browser compatible implementation of scrollIntoView. */
function scrollIntoView(e)
{
  window.scrollTo(0,e.offsetHeight + findPosY(e));
}

EOFEOF
      );
    //###############################################################
    elseif('carousel.js'==$name)
      fwrite($handle,<<<EOFEOF
var scroll_delay=500; // Delay before we start scrolling.
var scroll_delay_timer=false;
var scroll_interval=150; // Scroll speed.
var scroll_interval_timer=false;

/** Normalize index i to 0<=i<len */
function normalize(i)
{
  return (i+len) % len;
}

function carousel_up()
{
  var show=$('img_'+normalize(pos-1));
  var hide=$('img_'+normalize(pos+carousel_size-1));
  if(show)
  {
    move_before(show,$('img_'+pos));
    show.style.display='block';
  }
  if(hide)
    hide.style.display='none';
  pos=normalize(pos-1);
  document.forms['carousel'].pos.value=pos;
  if(!scroll_delay_timer)
    scroll_delay_timer=setTimeout('scr_on(carousel_up)',scroll_delay);
}

function carousel_down()
{
  var show=$('img_'+normalize(pos+carousel_size));
  if(show)
  {
    var hide=$('img_'+pos);
    move_before(show,$('more_down'));
    show.style.display='block';
    if(hide)
      hide.style.display='none';
    pos=normalize(pos+1);
    document.forms['carousel'].pos.value=pos;
    if(!scroll_delay_timer)
      scroll_delay_timer=setTimeout('scr_on(carousel_down)',scroll_delay);
  }
}

/** Starts scrolling. */
function scr_on(f)
{
  scroll_interval_timer=setInterval(f,scroll_interval);
}

/** Cancels scrolling. */
function scr_off()
{
  if(scroll_delay_timer)
  {
    clearTimeout(scroll_delay_timer);
    scroll_delay_timer=0;
  }
  if(scroll_interval_timer)
  {
    clearInterval(scroll_interval_timer);
    scroll_interval_timer=0;
  }
}

/** Translates a click on a thumbnail image into a POST request. */
function go(e)
{
  var carousel=document.forms['carousel'];
  if(carousel)
  {
    carousel.action=e.href;
    carousel.submit();
    return ONCLICK_OK;
  }
  else
    return ONCLICK_FAILED;
}


// TITLE EDITOR

function do_show_title_editor()
{
  var title_blck=$('title_block');
  var title_form=$('title_form');
  if(title_blck && title_form)
  {
    title_blck.style.display='none';
    title_form.style.display='block';
    title_form.title_txt.focus();
    return ONCLICK_OK;
  }
  return ONCLICK_FAILED;
}

function do_save_title()
{
  var title =$('title');
  var title_blck=$('title_block');
  var title_ctrl=$('title_ctrl');
  var title_form=$('title_form');
  var title_sub =$('title_sub');
  var title_busy=$('title_busy');
  if(title && title_blck && title_ctrl && title_form && title_sub && title_busy)
  {
    function update_title(xml)
      {
        title_sub.style.display ='inline';
        title_busy.style.display='none';
        if(report_alert(xml))
          return;
        title_blck.style.display='block';
        title_form.style.display='none';
        var xi=getFirstChildByTagName(xml,'image');
        if(xi)
        {
          var new_title=xi.getAttribute('title');
          title.innerHTML=new_title;
          document.title=new_title+' &laquo; '+xi.getAttribute('path');
          // Now set the carousel title.
          var ttl=$('ttl_'+img);
          ttl.innerHTML=new_title;
        }
      }
    if( ajax_form(title_form,{oncomplete:update_title}) )
    {
      title_sub.style.display='none';
      title_busy.style.display='inline';
      return ONCLICK_OK;
    }
    else
      alert('Error submitting new title.');
  }
  return ONCLICK_FAILED;
}

// CAPTION EDITOR

function do_show_caption_editor()
{
  var cap_para=$('cap_para');
  var cap_form=$('cap_form');
  if(cap_para && cap_form)
  {
    cap_form.cap_txt.style.height=''+Math.max(40,cap_para.offsetHeight)+'px';
    cap_para.style.display='none';
    cap_form.style.display='block';
    cap_form.cap_txt.focus();
    return ONCLICK_OK;
  }
  return ONCLICK_FAILED;
}

function do_save_caption()
{
  var caption =$('caption');
  var cap_para=$('cap_para');
  var cap_ctrl=$('cap_ctrl');
  var cap_form=$('cap_form');
  var cap_sub =$('cap_sub');
  var cap_busy=$('cap_busy');
  if(caption && cap_para && cap_ctrl && cap_form && cap_sub && cap_busy)
  {
    function update_caption(xml)
      {
        cap_para.style.display='block';
        cap_form.style.display='none';
        cap_sub.style.display ='inline';
        cap_busy.style.display='none';
        var xc=xml.getElementsByTagName('caption');
        if(xc)
        {
          var txt=getInnerText(xc[0]);
          caption.innerHTML=txt.replace(/\\n/g,'<br />');
          cap_ctrl.innerHTML=txt.length? 'edit': 'add caption';
          // Now set the carousel caption.
          var para=$('cap_'+img);
          xc=xml.getElementsByTagName('brief');
          para.innerHTML=getInnerText(xc[0]);
        }
      }
    if( ajax_form(cap_form,{oncomplete:update_caption}) )
    {
      cap_sub.style.display='none';
      cap_busy.style.display='inline';
      return ONCLICK_OK;
    }
    else
      alert('Error submitting comment.');
  }
  return ONCLICK_FAILED;
}

function do_show_comment_editor()
{
  var cmt_ctrl=$('cmt_ctrl');
  var cmt_form=$('cmt_form');
  if(cmt_ctrl && cmt_form)
  {
    cmt_ctrl.parentNode.style.display='none';
    cmt_form.style.display='block';
    scrollIntoView(cmt_form);
    cmt_form.cmt_author.focus();
    return ONCLICK_OK;
  }
  return ONCLICK_FAILED;
}

function update_num_comments(num)
{
  var nc=$('nc_'+img);
  if(num==0)
  {
    if(nc)
      fade_out(nc);
  }
  else
  {
    if(!nc)
    {
      nc=document.createElement('span');
      nc.id='nc_'+img;
      nc.className='num_comments';
      var dtc=$('dtc_'+img);
      fade_in(nc);
      dtc.insertBefore(nc,dtc.firstChild);
    }
    nc.innerHTML=''+num+'&nbsp;comment'+(num>1? 's': '');
  }
}

/** Construct a DIV from XML <comment> */
function div_comment(comment)
{
  var div=document.createElement('div');
  div.className='comment';
  var txt   =getInnerText( getFirstChildByTagName(comment,'text') );
  var author=getInnerText( getFirstChildByTagName(comment,'author') );
  var date  =getInnerText( getFirstChildByTagName(comment,'date') );
  div.innerHTML='<p>'+txt+'</p><p class="by">posted by <span class="author">'
    +author+'</span> on '+date+'</p>';
  return div;
}

/** Pop up an alert dialogue, if the album XML contains an <alert> element. */
function report_alert(album)
{
  var a=getFirstChildByTagName(album,'alert');
  if(a)
  {
    alert(getInnerText(a));
    return true;
  }
  return false;
}

function do_submit_comment()
{
  var cmt_head=$('cmt_head');
  var cmt_ctrl=$('cmt_ctrl');
  var cmt_form=$('cmt_form');
  var cmt_busy=$('cmt_busy');
  var cmt_sub =$('cmt_sub');
  if(cmt_ctrl && cmt_form && cmt_busy && cmt_sub)
  {
    function update_comment(xml)
      {
        cmt_sub.style.display='inline';
        cmt_busy.style.display='none';
        if(report_alert(xml))
          return;
        var cs=xml.getElementsByTagName('comment');
        for(var i=cs.length-1; i>=0; i--)
        {
          var id=cs[i].getAttribute('id');
          if($(id))
              break; // We already have this comment, so give up.
          else if(i==0)
              fade_in(cmt_head,'block'); // Make the Comment header visible.
          var div=div_comment(cs[i]);
          div.setAttribute('id',id);
          fade_in(div);
          cmt_form.parentNode.insertBefore(div,cmt_form);
        }
        update_num_comments(cs.length);
        cmt_form.reset();
        cmt_form.style.display='none';
        cmt_ctrl.parentNode.style.display='block';
      }
    if( ajax_form(cmt_form,{oncomplete:update_comment}) )
    {
      cmt_sub.style.display='none';
      cmt_busy.style.display='inline';
      return ONCLICK_OK;
    }
    else
      alert('Error submitting comment.');
  }
  return ONCLICK_FAILED;
}

function do_delete_comment(e)
{
  // Find comment.
  while(e && e.parentNode)
  {
    e=e.parentNode;
    if(e.className=='comment')
    {
      var id=e.getAttribute('id');
      if(id==null || id=='')
        break;
      ajax('?d=e&img='+img,{data:'do=delcomment&cid='+id});
      fade_out(e);
      // If there are no more comments, then hide the header, too.
      var cdivs=$('comments').getElementsByTagName('div');
      if(!(cdivs && cdivs.length>1))
          fade_out($('cmt_head'),true);
      update_num_comments(Math.max(0,cdivs.length-1));
      return ONCLICK_OK;
    }
  }
  return ONCLICK_FAILED;
}

EOFEOF
      );
    //###############################################################
    elseif('slideshow.js'==$name)
      fwrite($handle,<<<EOFEOF
function remove_element(e)
{
  //if(e && e.parentNode)
  //  e.parentNode.removeChild(e);
  // Alternatively...
  if(e)
    e.style.display='none';
}

//
// SlideShowNext
//

function SlideShowNext(idx,last,delay)
{
  // The image to load
  this.idx= (idx+ss.len) % ss.len;
  // Last image in a slide-show, or -1.
  this.last=dflt(last,-1);
  // Minumum delay before displaying it.
  this.delay=dflt(delay,0);

  this.id=false;
  this.timer=false;
  this.req=false; // XMLHttpRequest object
  this.xml=false; // XML that describes the next image.
  this.img=false;
  this.img_complete=false; // img.complete doesn't work on Safari.

  // Event Handlers
  var self=this;

  this.evt_req_complete =function()
  {
    if(self.req && self.req.readyState==4)
    {
      if(self.req.status==200)
      {
        self.xml=
          self.req.responseXML.documentElement.getElementsByTagName('image');
        if(self.xml)
        {
          self.xml=self.xml[0];
          self.create_img();
        }
      }
      self.req=false;
    }
  }
  this.evt_img_onload =function(e)
  {
    if(e.currentTarget)   e=e.currentTarget; // Mozilla/Safari/w3c
    else if(window.event) e=window.event.srcElement; // IE

    if(self.img==e)
    {
      self.img_complete=true;   
      self.show();
    }
  }
  this.evt_timer =function()
  {
    self.timer=false;
    self.show();
  }
} // end SlideShowNext

SlideShowNext.prototype.load=function()
{
  this.id='img_'+this.idx+'_'+ss.isize;
  this.timer=setTimeout(this.evt_timer,this.delay);
  this.req=new XMLHttpRequest();
  if(this.req)
  {
    this.req.open('GET','?d=x&img='+this.idx+'&isize='+ss.isize,true);
    this.req.setRequestHeader(
      'If-Modified-Since','Sat, 1 Jan 2000 00:00:00 GMT');
    this.req.onreadystatechange=this.evt_req_complete;
    this.req.send(null);
  }
  else
  {
    this.cancel();
    return;
  }
  document.getElementById('busy').style.display='inline';
}

SlideShowNext.prototype.cancel=function()
{
  if(this.timer)
    clearTimeout(this.timer);
  if(this.img)
    remove_element(this.img);
  if(this.req)
    this.req.abort();
  if(this==ss.next)
    ss.next=false;
}

/** Utility function, called by evt_req_complete(). Creates the img element. */
SlideShowNext.prototype.create_img=function()
{
  this.img=document.getElementById(this.id);
  if(this.img)
  {
    this.img_complete=true; // not necessarily true, but probably OK.
    this.show();
  }
  else
  {
    // Create a new image.
    this.img=document.createElement('img');
    this.img.style.display='none';
    document.getElementById('single').appendChild(this.img);
    this.img.setAttribute('id',this.id);
    addEvent(this.img,'load',this.evt_img_onload);
    this.img.setAttribute('src',this.xml.getAttribute('src'));
    this.img.setAttribute('width',this.xml.getAttribute('width'));
    this.img.setAttribute('height',this.xml.getAttribute('height'));
  }
}

SlideShowNext.prototype.show=function()
{
  // Ready when timeout has expired and the img is loaded.
  if( ss.next!=this || this.timer || !this.xml || !this.img_complete)
    return;

  // Show the image
  remove_element( document.getElementById(ss.curr_id) );
  this.img.style.display='block';
  // Change the title
  document.title=this.xml.getAttribute('title');
  document.getElementById('title').innerHTML=document.title;
  // Change the download link
  document.getElementById('download').href='?dl&img='+this.idx;
  // Change the nav links
  document.getElementById('nav_t').href='?d=t&img='+this.idx;
  document.getElementById('nav_c').href='?d=c&img='+this.idx;
  document.getElementById('busy').style.display='none';
  // Clean up and get the next one...
  var stop=( this.idx!=ss.curr && this.idx==this.last );
  ss.curr=this.idx;
  ss.curr_id=this.id;
  ss.next=false;
  if(stop)
  {
    ss.do_stop();
  }
  else if(this.last>=0)
  {
    ss.next=new SlideShowNext(ss.curr+1,this.last,ss.delay);
    ss.next.load()
  }
}

//
//  SlideShow
//

function SlideShow(curr,len,isize,delay)
{
  this.curr=curr;
  this.curr_id='img_'+curr+'_'+isize;

  this.len=len;
  this.next=false;
  // The desired image size (may not always be the size of the current image).
  this.isize=isize;

  this.delay=dflt(delay,4000);

  // constants
  this.min_isize=0;
  this.max_isize=5;
}

SlideShow.prototype.do_prev=function()
{
  this.do_next(-1);
}

SlideShow.prototype.do_next=function(offset)
{
  offset=dflt(offset,1);
  var next=new SlideShowNext(this.curr+offset);
  if(this.next)
  {
    if(this.next.last>=0)
      return; // Playing slide show - ignore.
    if(this.next.idx==next.idx)
      return; // already loading this one (for some reason).
    this.next.cancel()
  }
  this.next=next;
  this.next.load()
}

SlideShow.prototype.do_play=function()
{
  var next=new SlideShowNext(this.curr+1,this.curr,this.delay);
  if(this.next)
  { 
    if(this.next.idx==next.idx)
      return; // already loading this one
    this.next.cancel();
  }
  this.next=next;
  this.next.load()
  // Set control to STOP
  document.getElementById('play').style.display='none';
  document.getElementById('stop').style.display='inline';
  document.getElementById('prev').className='off';
  document.getElementById('next').className='off';
}

SlideShow.prototype.do_stop=function()
{
  if(this.next)
    this.next.cancel();
  document.getElementById('busy').style.display='none';
  document.getElementById('play').style.display='inline';
  document.getElementById('stop').style.display='none';
  document.getElementById('prev').className='on';
  document.getElementById('next').className='on';
}

SlideShow.prototype.do_bigger=function()
{
  if(this.isize < this.max_isize)
    ss.resize(this.isize+1);
}

SlideShow.prototype.do_smaller=function()
{
  if(this.isize > this.min_isize)
    ss.resize(this.isize-1);
}


/** Utility function called by do_bigger() & do_smaller(). */
SlideShow.prototype.resize=function(isize)
{
  this.isize=isize;
  // Store isize to a cookie - this tells the server which size image to send.
  document.cookie=('isize='+this.isize+'; path=/');
  // Load the new image.
  var last=-1;
  if(this.next)
  {
    last=this.next.last;
    this.next.cancel();
  }
  this.next=new SlideShowNext(this.curr,last);
  this.next.load()

  // toggle size icons
  var smaller=document.getElementById('smaller');
  if(this.isize > this.min_isize)
    smaller.className='on';
  else
    smaller.className='off';

  var bigger=document.getElementById('bigger');
  if(this.isize < this.max_isize)
    bigger.className='on';
  else
    bigger.className='off';
}


EOFEOF
      );
    //###############################################################
    elseif('transparent.gif'==$name)
      fwrite($handle,base64_decode(<<<EOFEOF
R0lGODlhAQABAIAAAP///wAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw==
====
EOFEOF
      ));
    //###############################################################
    elseif('themes/black.css'==$name)
      fwrite($handle,<<<EOFEOF
body {
 background-color:black;
 color:white;
}

body div a:hover { color:white; }
a:link { color:#f60; }
a:active { color:#cc0; }
a:visited { color:#f60; }

#path,
#pages,
#pages span.off,
#browser,
#header a.control,
#main a.control,
input,
textarea,
button {
 background-color:#222;
 border-color:#666;
 color:white;
}

.iconbox a:hover img,
#pages a,
#browser div.here {
 background-color:#444;
 border-color:#666;
 color:white;
}

#pages a:hover {
 background-color:black;
}

.dir td.thumb img {
 border:solid 1px white;
}

EOFEOF
      );
    //###############################################################
    elseif('themes/neutral.css'==$name)
      fwrite($handle,<<<EOFEOF
body {
 background-color:#999;
}

.iconbox a:hover img,
#browser div.here,
#header a.control,
#main a.control,
button {
 background-color:#e8e8ff;
 border-color:#44b;
}

div#browser {
 border-color:#666;
}

.dir td.thumb img {
 border:solid 1px white;
}

.exif,
.comment p.by,
.date_time,
.num_comments {
 color:#444;
}

EOFEOF
      );
    //###############################################################
    else
        die("Unknown file: $name");
    fclose($handle);
  }
  return htmlspecialchars($result);
}

?>
